//
//  BGM700View.swift
//  MicroLifeDeviceSDK-Demo_Swift
//
//  Created by 吳憲有 on 2024/5/31.
//

import SwiftUI

struct BGM700View: View {
    @Environment(AppInfo.self) var appInfo
    
    @Environment(\.horizontalSizeClass) var horizontalSizeClass
    
    @Environment(\.verticalSizeClass) var verticalSizeClass
    
    var viewModel = BGM700ViewModel()
    
    var menuInfo: MenuInfo
    
    var body: some View {
        let orientation = DeviceOrientation.checkOrientation(horizontalSizeClass: horizontalSizeClass, verticalSizeClass: verticalSizeClass)
        ZStack {
            switch orientation {
            case .iPhoneHorizontal, .OtherDevice:
                HorizontalLayoutView(scale: 0.85) {
                    viewModel.startScan()
                } cancelScanAction: {
                    viewModel.cancelScan()
                } disconnectDeviceAction: {
                    viewModel.disconnectDevice()
                } commandViewContent: {
                    CustomBGM700View(viewModel: viewModel)
                }
            case .iPhoneVertical, .iPad:
                VerticalLayoutView(scale: 0.75) {
                    viewModel.startScan()
                } cancelScanAction: {
                    viewModel.cancelScan()
                } disconnectDeviceAction: {
                    viewModel.disconnectDevice()
                } commandViewContent: {
                    CustomBGM700View(viewModel: viewModel)
                }
            }
        }
        .toolbar {
            CustomToolbarItem(
                filePath: log.filePath,
                fileName: log.fileName,
                refreshAction: {
                    viewModel.refreshData()
                })
        }
        .navigationTitle(menuInfo.title)
        .padding(.horizontal, 8)
        .onChange(of: appInfo.showViewType) {
            if appInfo.showViewType == .bgm700 {
                viewModel.setupBLESDK()
            }
        }
        .onAppear {
            appInfo.showViewType = .bgm700
            viewModel.setupBLESDK()
        }
        .onDisappear {
            appInfo.showViewType = .menu
            viewModel.disconnectDevice()
        }
    }
}

struct CustomBGM700View: View {
    var viewModel: BGM700ViewModel
    
    @State private var param = PARAM.all
    
    @State private var sn1 = ""
    
    @State private var sn2 = ""
    
    var body: some View {
        VStack(alignment: .trailing, spacing: 16) {
            HStack {
                Text("PARAM:")
                PARAMPicker(param: $param) {
                    viewModel.param = param
                }
            }
            HStack {
                TextField("SN1", text: $sn1, onCommit: {
                    if let intValue = Int(sn1) {
                        viewModel.sn1 = intValue
                    }
                    // Dismiss the keyboard
                    UIApplication.shared.sendAction(#selector(UIResponder.resignFirstResponder), to: nil, from: nil, for: nil)
                })
                .textFieldStyle(RoundedBorderTextFieldStyle())
                .keyboardType(.numberPad)
                TextField("SN2", text: $sn2, onCommit: {
                    if let intValue = Int(sn2) {
                        viewModel.sn2 = intValue
                    }
                    // Dismiss the keyboard
                    UIApplication.shared.sendAction(#selector(UIResponder.resignFirstResponder), to: nil, from: nil, for: nil)
                })
                .textFieldStyle(RoundedBorderTextFieldStyle())
                .keyboardType(.numberPad)
            }
            Button(action: {
                viewModel.queryMachineFunction()
            }) {
                Text("Query Machine Function")
            }
            Button(action: {
                viewModel.queryBloodGlucoseData()
            }) {
                Text("Query Blood Glucose Data")
            }
            Button(action: {
                viewModel.downloadBloodGlucoseData()
            }) {
                Text("Download Blood Glucose Data")
            }
            Button(action: {
                viewModel.queryUricAcidData()
            }) {
                Text("Query Uric Acid Data")
            }
            Button(action: {
                viewModel.downloadUricAcidData()
            }) {
                Text("Download Uric Acid Data")
            }
            Button(action: {
                viewModel.queryCholesterolData()
            }) {
                Text("Query Cholesterol Data")
            }
            Button(action: {
                viewModel.downloadCholesterolData()
            }) {
                Text("Download Cholesterol Data")
            }
            Button(action: {
                viewModel.queryBloodKetoneData()
            }) {
                Text("Query Blood Ketone Data")
            }
            Button(action: {
                viewModel.downloadBloodKetoneData()
            }) {
                Text("Download Blood Ketone Data")
            }
            Button(action: {
                viewModel.queryGkiData()
            }) {
                Text("Query GKI Data")
            }
            Button(action: {
                viewModel.downloadGkiData()
            }) {
                Text("Download GKI Data")
            }
            Button(action: {
                viewModel.queryLactateData()
            }) {
                Text("Query Lactate Data")
            }
            Button(action: {
                viewModel.downloadLactateData()
            }) {
                Text("Download Lactate Data")
            }
            Button(action: {
                viewModel.queryHemeData()
            }) {
                Text("Query Heme Data")
            }
            Button(action: {
                viewModel.downloadHemeData()
            }) {
                Text("Download Heme Data")
            }
        }
    }
}

#Preview {
    BGM700View(menuInfo: MenuInfo())
}
