//
//  BGM700ViewModel.swift
//  MicroLifeDeviceSDK-Demo_Swift
//
//  Created by 吳憲有 on 2024/5/31.
//

import Foundation

@Observable
class BGM700ViewModel {
    var deviceConnected = bgm700.deviceConnected
    
    var param = PARAM.all
    
    var sn1 = 0
    
    var sn2 = 0
    
    func setupBLESDK() {
        log.clear()

        sdk.cancelAllConnect()
        
        bgm700.autoConnect(true)

        bgm700.getDidUpdateStateBlock { state in
            log.T(text:"isBLEStatePoweredOn: \(state)")
        } scanDeviceBlock: { checkMacType, device in
            if let microLifeDevice = device as? MicroLifeDevice {
                log.T(text: "ScanDevice:\nName:\(microLifeDevice.name) MacType:\(checkMacType.description) RSSI:\(microLifeDevice.rssi)\nUUID:\(microLifeDevice.uuid)")
            }
        } cancelScanBlock: {
            log.T(text:"Cancel Scan")
        } connectDeviceStateBlock: { device, central, peripheral, state, error in
            log.T(text:"Device: \(device)\nPeripheral State: \(state)")
            log.E(error: error)
        } cancelAllDevicesConnectionBlock: {
            log.T(text:"Cancel All Devices Connection")
            bgm700.startScan()
        }
        
        bgm700.deviceConnectedBlock {
            self.deviceConnected = bgm700.deviceConnected
        }
        
        /// bloodSugar
        bgm700.getReadDataModelBlock { CMD, model, error in
            log.M(CMD: CMD, model: model)
            log.E(error: error)
            guard let command = CMDBGM700(rawValue: CMD) else {
                return
            }
            switch command {
            case .queryMachineFunction:
                // Handle query machine function
                self.setSN1(model: model as! MicroLifeDeviceInfo)
                break
            case .queryBloodGlucoseDatas:
                // Handle query blood glucose data
                self.setSN1(model: model as! MicroLifeDeviceInfo)
                break
            case .queryUricAcidDatas:
                // Handle query uric acid data
                self.setSN1(model: model as! MicroLifeDeviceInfo)
                break
            case .queryCholesterolDatas:
                // Handle query cholesterol data
                self.setSN1(model: model as! MicroLifeDeviceInfo)
                break
            case .queryBloodKetoneDatas:
                // Handle query blood ketone data
                self.setSN1(model: model as! MicroLifeDeviceInfo)
                break
            case .queryGKIDatas:
                // Handle query GKI data
                self.setSN1(model: model as! MicroLifeDeviceInfo)
                break
            case .queryLactateDatas:
                // Handle query lactate data
                self.setSN1(model: model as! MicroLifeDeviceInfo)
                break
            case .queryHemeDatas:
                // Handle query heme data
                self.setSN1(model: model as! MicroLifeDeviceInfo)
                break
            case .downloadBloodGlucoseData:
                // Handle download blood glucose data
                break
            case .downloadUricAcidData:
                // Handle download uric acid data
                break
            case .downloadCholesterolData:
                // Handle download cholesterol data
                break
            case .downloadBloodKetoneData:
                // Handle download blood ketone data
                break
            case .downloadGKIData:
                // Handle download GKI data
                break
            case .downloadLactateData:
                // Handle download lactate data
                break
            case .downloadHemeData:
                // Handle download heme data
                break
            default:
                // Handle other cases
                break
            }
        }
    }
    
    func setSN1(model: MicroLifeDeviceInfo) {
        self.sn1 = model.maxMemory.intValue
    }

    func refreshData() {
        log.clear()
        log.T(text:"refreshData")
        disconnectDevice()
        startScan()
    }
    
    func startScan() {
        bgm700.startScan()
        log.T(text: "Start Scan")
    }

    func cancelScan() {
        bgm700.cancelScan()
        log.T(text: "Cancel Scan")
    }
    
    func disconnectDevice() {
        if deviceConnected {
            bgm700.disconnectDevice()
        }
        log.T(text: deviceConnected ? "unConnect" : "disconnectDevice")
    }
    
    func queryMachineFunction() {
        bgm700.queryMachineFunction()
        log.T(text: "Query Machine Function")
    }
    
    func queryBloodGlucoseData() {
        bgm700.queryBloodGlucoseDatas(MicroLifeDeviceSDK.PARAM(rawValue: param.rawValue) ?? MicroLifeDeviceSDK.PARAM.all, sn1: NSNumber(value: sn1), sn2: NSNumber(value: sn2))
        log.T(text: "Query Blood Glucose Data with PARAM: \(param), SN1: \(sn1), SN2: \(sn2)")
    }
    
    func downloadBloodGlucoseData() {
        bgm700.downloadBloodGlucoseData(MicroLifeDeviceSDK.PARAM(rawValue: param.rawValue) ?? MicroLifeDeviceSDK.PARAM.all, sn1: NSNumber(value: sn1), sn2: NSNumber(value: sn2))
        log.T(text: "Download Blood Glucose Data with PARAM: \(param), SN1: \(sn1), SN2: \(sn2)")
    }
    
    func queryUricAcidData() {
        bgm700.queryUricAcidDatas(MicroLifeDeviceSDK.PARAM(rawValue: param.rawValue) ?? MicroLifeDeviceSDK.PARAM.all, sn1: NSNumber(value: sn1), sn2: NSNumber(value: sn2))
        log.T(text: "Query Uric Acid Data with PARAM: \(param), SN1: \(sn1), SN2: \(sn2)")
    }
    
    func downloadUricAcidData() {
        bgm700.downloadUricAcidData(MicroLifeDeviceSDK.PARAM(rawValue: param.rawValue) ?? MicroLifeDeviceSDK.PARAM.all, sn1: NSNumber(value: sn1), sn2: NSNumber(value: sn2))
        log.T(text: "Download Uric Acid Data with PARAM: \(param), SN1: \(sn1), SN2: \(sn2)")
    }
    
    func queryCholesterolData() {
        bgm700.queryCholesterolDatas(MicroLifeDeviceSDK.PARAM(rawValue: param.rawValue) ?? MicroLifeDeviceSDK.PARAM.all, sn1: NSNumber(value: sn1), sn2: NSNumber(value: sn2))
        log.T(text: "Query Cholesterol Data with PARAM: \(param), SN1: \(sn1), SN2: \(sn2)")
    }
    
    func downloadCholesterolData() {
        bgm700.downloadCholesterolData(MicroLifeDeviceSDK.PARAM(rawValue: param.rawValue) ?? MicroLifeDeviceSDK.PARAM.all, sn1: NSNumber(value: sn1), sn2: NSNumber(value: sn2))
        log.T(text: "Download Cholesterol Data with PARAM: \(param), SN1: \(sn1), SN2: \(sn2)")
    }
    
    func queryBloodKetoneData() {
        bgm700.queryBloodKetoneDatas(MicroLifeDeviceSDK.PARAM(rawValue: param.rawValue) ?? MicroLifeDeviceSDK.PARAM.all, sn1: NSNumber(value: sn1), sn2: NSNumber(value: sn2))
        log.T(text: "Query Blood Ketone Data with PARAM: \(param), SN1: \(sn1), SN2: \(sn2)")
    }
    
    func downloadBloodKetoneData() {
        bgm700.downloadBloodKetoneData(MicroLifeDeviceSDK.PARAM(rawValue: param.rawValue) ?? MicroLifeDeviceSDK.PARAM.all, sn1: NSNumber(value: sn1), sn2: NSNumber(value: sn2))
        log.T(text: "Download Blood Ketone Data with PARAM: \(param), SN1: \(sn1), SN2: \(sn2)")
    }
    
    func queryGkiData() {
        bgm700.queryGKIDatas(MicroLifeDeviceSDK.PARAM(rawValue: param.rawValue) ?? MicroLifeDeviceSDK.PARAM.all, sn1: NSNumber(value: sn1), sn2: NSNumber(value: sn2))
        log.T(text: "Query GKI Data with PARAM: \(param), SN1: \(sn1), SN2: \(sn2)")
    }
    
    func downloadGkiData() {
        bgm700.downloadGKIData(MicroLifeDeviceSDK.PARAM(rawValue: param.rawValue) ?? MicroLifeDeviceSDK.PARAM.all, sn1: NSNumber(value: sn1), sn2: NSNumber(value: sn2))
        log.T(text: "Download GKI Data with PARAM: \(param), SN1: \(sn1), SN2: \(sn2)")
    }
    
    func queryLactateData() {
        bgm700.queryLactateDatas(MicroLifeDeviceSDK.PARAM(rawValue: param.rawValue) ?? MicroLifeDeviceSDK.PARAM.all, sn1: NSNumber(value: sn1), sn2: NSNumber(value: sn2))
        log.T(text: "Query Lactate Data with PARAM: \(param), SN1: \(sn1), SN2: \(sn2)")
    }
    
    func downloadLactateData() {
        bgm700.downloadLactateData(MicroLifeDeviceSDK.PARAM(rawValue: param.rawValue) ?? MicroLifeDeviceSDK.PARAM.all, sn1: NSNumber(value: sn1), sn2: NSNumber(value: sn2))
        log.T(text: "Download Lactate Data with PARAM: \(param), SN1: \(sn1), SN2: \(sn2)")
    }
    
    func queryHemeData() {
        bgm700.queryHemeDatas(MicroLifeDeviceSDK.PARAM(rawValue: param.rawValue) ?? MicroLifeDeviceSDK.PARAM.all, sn1: NSNumber(value: sn1), sn2: NSNumber(value: sn2))
        log.T(text: "Query Heme Data with PARAM: \(param), SN1: \(sn1), SN2: \(sn2)")
    }
    
    func downloadHemeData() {
        bgm700.downloadHemeData(MicroLifeDeviceSDK.PARAM(rawValue: param.rawValue) ?? MicroLifeDeviceSDK.PARAM.all, sn1: NSNumber(value: sn1), sn2: NSNumber(value: sn2))
        log.T(text: "Download Heme Data with PARAM: \(param), SN1: \(sn1), SN2: \(sn2)")
    }
    
}
