//
//  BGM800ViewModel.swift
//  MicroLifeDeviceSDK-Demo_Swift
//
//  Created by 吳憲有 on 2024/5/31.
//

import Foundation

@Observable
class BGM800ViewModel {
    var deviceConnected = bgm800.deviceConnected
    
    var BTModuleName = ""
    
    var recordCounts = 0
    
    var selectedRecordCount = 0

    func setupBLESDK() {
        log.clear()

        sdk.cancelAllConnect()
        
        bgm800.autoConnect(true)

        bgm800.getDidUpdateStateBlock { state in
            log.T(text:"isBLEStatePoweredOn: \(state)")
        } scanDeviceBlock: { checkMacType, device in
            if let microLifeDevice = device as? MicroLifeDevice {
                log.T(text: "ScanDevice:\nName:\(microLifeDevice.name) MacType:\(checkMacType.description) RSSI:\(microLifeDevice.rssi)\nUUID:\(microLifeDevice.uuid)")
            }
        } cancelScanBlock: {
            log.T(text:"Cancel Scan")
        } connectDeviceStateBlock: { device, central, peripheral, state, error in
            log.T(text:"Device: \(device)\nPeripheral State: \(state)")
            log.E(error: error)
        } cancelAllDevicesConnectionBlock: {
            log.T(text:"Cancel All Devices Connection")
            bgm700.startScan()
        }
        
        bgm800.deviceConnectedBlock {
            self.deviceConnected = bgm800.deviceConnected
        }
        
        /// bloodSugar
        bgm800.getReadDataModelBlock { CMD, model, error in
            log.M(CMD: CMD, model: model)
            log.E(error: error)
            switch CMD {
            default:
                // Handle other cases if needed
                break
            }
        }
    }

    func refreshData() {
        log.clear()
        log.T(text:"refreshData")
        disconnectDevice()
        startScan()
    }
    
    func startScan() {
        bgm800.startScan()
        log.T(text: "Start Scan")
    }

    func cancelScan() {
        bgm800.cancelScan()
        log.T(text: "Cancel Scan")
    }
    
    func disconnectDevice() {
        if deviceConnected {
            bgm800.disconnectDevice()
        }
        log.T(text: deviceConnected ? "unConnect" : "disconnectDevice")
    }
    
    // New Methods
    func readDeviceName() {
        bgm800.readDeviceName()
        log.T(text: "Read Device Name")
    }

    func readFWVersion() {
        bgm800.readFWVersion()
        log.T(text: "Read FW Version")
    }

    func getBLEStatus() {
        bgm800.getBLEStatus()
        log.T(text: "Get BLE Status")
    }

    func readLastData() {
        bgm800.readLastData()
        log.T(text: "Read Last Data")
    }

    func readData() {
        bgm800.readData(selectedRecordCount)
        log.T(text: "Read Data with Counts: \(selectedRecordCount)")
    }

    func readDataLength() {
        bgm800.readDataLength()
        log.T(text: "Read Data Length")
    }

    func uploadMeterData() {
        bgm800.uploadMeterData()
        log.T(text: "Upload Meter Data")
    }

    func disconnect() {
        bgm800.disconnect()
        log.T(text: "Disconnect")
    }

    func readDateTime() {
        bgm800.readDateTime()
        log.T(text: "Read DateTime")
    }

    func writeDeviceName() {
        bgm800.writeDeviceName(BTModuleName)
        log.T(text: "Write Device Name: \(BTModuleName)")
    }

    func getMeterDataTransmissionStatus() {
        bgm800.getMeterDataTransmissionStatus()
        log.T(text: "Get Meter Data Transmission Status")
    }

    func getCommunicationMode() {
        bgm800.getCommunicationMode()
        log.T(text: "Get Communication Mode")
    }

    func setBLEParingMode() {
        bgm800.setBLEParingMode()
        log.T(text: "Set BLE Paring Mode")
    }

    func setMeterAuthorization() {
        bgm800.setMeterAuthorization()
        log.T(text: "Set Meter Authorization")
    }

    func setParingRequest() {
        bgm800.setParingRequest()
        log.T(text: "Set Paring Request")
    }

    func setParingTimeOut() {
        bgm800.setParingTimeOut()
        log.T(text: "Set Paring TimeOut")
    }
}
