//
//  BP3gView.swift
//  MicroLifeDeviceSDK-Demo_Swift
//
//  Created by 吳憲有 on 2024/5/8.
//

import SwiftUI

struct BP3gView: View {
    @Environment(AppInfo.self) var appInfo

    @Environment(\.horizontalSizeClass) var horizontalSizeClass

    @Environment(\.verticalSizeClass) var verticalSizeClass

    var viewModel = BP3gViewModel()

    var menuInfo: MenuInfo

    var body: some View {
        let orientation = DeviceOrientation.checkOrientation(horizontalSizeClass: horizontalSizeClass, verticalSizeClass: verticalSizeClass)
        ZStack {
            switch orientation {
                case .iPhoneHorizontal, .OtherDevice:
                    HorizontalLayoutView(scale: 0.6) {
                        viewModel.startScan()
                    } cancelScanAction: {
                        viewModel.cancelScan()
                    } disconnectDeviceAction: {
                        viewModel.disconnectDevice()
                    } commandViewContent: {
                        CustomBP3gView(viewModel: viewModel)
                    }
                case .iPhoneVertical, .iPad:
                    VerticalLayoutView(scale: 0.75) {
                        viewModel.startScan()
                    } cancelScanAction: {
                        viewModel.cancelScan()
                    } disconnectDeviceAction: {
                        viewModel.disconnectDevice()
                    } commandViewContent: {
                        CustomBP3gView(viewModel: viewModel)
                    }
            }
        }
        .toolbar {
            CustomToolbarItem(
                filePath: log.filePath,
                fileName: log.fileName,
                refreshAction: {
                    viewModel.refreshData()
                })
        }
        .navigationTitle(menuInfo.title)
        .padding(.horizontal, 8)
        .onChange(of: appInfo.showViewType) {
            if appInfo.showViewType == .bp3g {
                viewModel.setupBLESDK()
            }
        }
        .onAppear {
            appInfo.showViewType = .bp3g
            viewModel.setupBLESDK()
        }
        .onDisappear {
            appInfo.showViewType = .menu
            viewModel.disconnectDevice()
        }
    }
}

struct CustomBP3gView: View {
    var viewModel: BP3gViewModel

    var body: some View {
        VStack(alignment: .trailing, spacing: 16) {
            Button(action: {
                viewModel.readHistorys()
            }) {
                Text("Read Historys")
            }
            Button(action: {
                viewModel.clearAllHistorys()
            }) {
                Text("Clear All Historys")
            }
            Button(action: {
                viewModel.disconnect()
            }) {
                Text("Disconnect")
            }
            Button(action: {
                viewModel.readUserAndVersionData()
            }) {
                Text("Read User And Version Data")
            }
            Button(action: {
                viewModel.writeUserData()
            }) {
                Text("Write User Data")
            }
            Button(action: {
                viewModel.readLastData()
            }) {
                Text("Read Last Data")
            }
            Button(action: {
                viewModel.clearLastData()
            }) {
                Text("Clear Last Data")
            }
            Button(action: {
                bp3g.readDeviceInfo()
            }) {
                Text("Read Device Info")
            }
        }
        .padding()
    }
}

#Preview {
    BP3gView(menuInfo: MenuInfo())
}
