//
//  BP3gViewModel.swift
//  MicroLifeDeviceSDK-Demo_Swift
//
//  Created by 吳憲有 on 2024/5/8.
//

import Foundation

@Observable
class BP3gViewModel {
    
    var deviceConnected = bp3g.deviceConnected

    func setupBLESDK() {
        
        log.clear()

        sdk.cancelAllConnect()

        bp3g.autoConnect(true)

        bp3g.getDidUpdateStateBlock { state in
            log.T(text: "isBLEStatePoweredOn: \(state)")
        } scanDeviceBlock: { checkMacType, device in
            if let microLifeDevice = device as? MicroLifeDevice {
                log.T(text: "ScanDevice:\nName:\(microLifeDevice.name) MacType:\(checkMacType.description) RSSI:\(microLifeDevice.rssi)\nUUID:\(microLifeDevice.uuid)")
            }
        } cancelScanBlock: {
            log.T(text: "Cancel Scan")
        } connectDeviceStateBlock: { device, _, _, state, error in
            log.T(text: "Device: \(device)\nPeripheral State: \(state)")
            log.E(error: error)
        } cancelAllDevicesConnectionBlock: {
            log.T(text: "Cancel All Devices Connection")
        }

        bp3g.getReadDataModelBlock { CMD, model, error in
            log.M(CMD: CMD, model: model)
            log.E(error: error)
            guard let command = CMD3G(rawValue: CMD) else {
                // Handle the case when CMD does not match any enumeration value
                // For example, log an error or perform other error-handling tasks
                return
            }
            switch command {
            case .readHistorys:
                // Handle reading history data
                break
            case .clearAllHistorys:
                // Handle clearing all history data
                break
            case .disconnect:
                // Handle disconnecting
                break
            case .readUserAndVersionData:
                // Handle reading user and version data
                break
            case .writeUser:
                // Handle writing user data
                break
            case .readLastData:
                // Handle reading last data
                break
            case .clearLastData:
                // Handle clearing last data
                break
            case .readDeviceInfo:
                // Handle reading device information
                break
            default:
                // Handle other cases if needed
                // For example, log an error or perform other error-handling tasks
                break
            }
        }

        bp3g.getValidationErrorBlock { item, info, value in
            log.T(text: "\(item): info:\n\(info) value:\n\(value)")
        }
    }

    func refreshData() {
        log.clear()
        log.T(text: "refreshData")
        disconnectDevice()
        startScan()
    }

    func startScan() {
        bp3g.startScan()
        log.T(text: "Start Scan")
    }

    func cancelScan() {
        bp3g.cancelScan()
        log.T(text: "Cancel Scan")
    }

    func disconnectDevice() {
        if deviceConnected {
            bp3g.disconnect()
            bp3g.disconnectDevice()
        }
        log.T(text: deviceConnected ? "unConnect" : "disconnectDevice")
    }

    /// bp 3g command
    func readHistorys() {
        bp3g.readHistorys()
        log.T(text: "Reading Historys")
    }

    func clearAllHistorys() {
        bp3g.clearAllHistorys()
        log.T(text: "Clearing All Historys")
    }

    func disconnect() {
        bp3g.disconnect()
        log.T(text: "Disconnecting")
    }

    func readUserAndVersionData() {
        bp3g.readUserAndVersionData()
        log.T(text: "Reading User And Version Data")
    }

    func writeUserData() {
        let ID = getNumAndLetterID(length: 11)
        let age = getAge()
        appInfo.user.bpmUserID = ID;
        appInfo.user.age = age as NSNumber;
        bp3g.writeUserData(ID, age: age)
        log.T(text: "Writing User Data：\(ID) Age: \(age)")
    }

    func readLastData() {
        bp3g.readLastData()
        log.T(text: "Reading Last Data")
    }

    func clearLastData() {
        bp3g.clearLastData()
        log.T(text: "Clearing Last Data")
    }

    func readDeviceInfo() {
        bp3g.readDeviceInfo()
        log.T(text: "Reading Device Info")
    }
}
