//
//  BP4gViewModel.swift
//  MicroLifeDeviceSDK-Demo_Swift
//
//  Created by 吳憲有 on 2024/5/10.
//

import Foundation

@Observable
class BP4gViewModel {

    var deviceConnected = bp4g.deviceConnected
    
    func setupBLESDK() {
        
        log.clear()

        sdk.cancelAllConnect()
        
        bp4g.autoConnect(true)

        bp4g.getDidUpdateStateBlock { state in
            log.T(text:"isBLEStatePoweredOn: \(state)")
        } scanDeviceBlock: { checkMacType, device in
            if let microLifeDevice = device as? MicroLifeDevice {
                log.T(text: "ScanDevice:\nName:\(microLifeDevice.name) MacType:\(checkMacType.description) RSSI:\(microLifeDevice.rssi)\nUUID:\(microLifeDevice.uuid)")
            }
        } cancelScanBlock: {
            log.T(text:"Cancel Scan")
        } connectDeviceStateBlock: { device, central, peripheral, state, error in
            log.T(text:"Device: \(device)\nPeripheral State: \(state)")
        } cancelAllDevicesConnectionBlock: {
            log.T(text:"Cancel All Devices Connection")
        }
        
        bp4g.deviceConnectedBlock {
            self.deviceConnected = bp4g.deviceConnected
        }
        
        bp4g.getReadDataModelBlock { CMD, model, error in
            log.M(CMD: CMD, model: model)
            log.E(error: error)
            guard let command = CMD4G(rawValue: CMD) else {
                // Handle the case when CMD does not match any enumeration value
                // For example, log an error or perform other error-handling tasks
                return
            }
            switch command {
            case .readHistorys:
                // Handle reading history data
                break
            case .clearAllHistorys:
                // Handle clearing all history data
                break
            case .disconnect:
                // Handle disconnecting
                break
            case .readUserAndVersionData:
                // Handle reading user and version data
                break
            case .writeUser:
                // Handle writing user data
                break
            case .readLastData:
                // Handle reading last data
                break
            case .clearLastData:
                // Handle clearing last data
                break
            case .readDeviceInfo:
                // Handle reading device information
                break
            case .readDeviceTime:
                // Handle reading device time
                break
            case .syncTiming:
                // Handle synchronizing timing
                break
            case .checkTransmit:
                // Handle checking transmit
                break
            case .readSerialNumber:
                // Handle reading serial number
                break
            default:
                // Handle other cases if needed
                // For example, log an error or perform other error-handling tasks
                break
            }
        }
        
        bp4g.getValidationErrorBlock { item, info, value in
            log.T(text: "\(item)\ninfo:\(info)\nvalue:\(value)")
        }
        
        bp4g.getPairingErrorBlock { error in
            log.T(text: "Pairing Error:\(error)")
        }
    }
    
    func refreshData() {
        log.clear()
        log.T(text:"refreshData")
        disconnectDevice()
        startScan()
    }
    
    func startScan() {
        bp4g.startScan()
        log.T(text: "Start Scan")
    }

    func cancelScan() {
        bp4g.cancelScan()
        log.T(text: "Cancel Scan")
    }
    
    func disconnectDevice() {
        if deviceConnected {
            bp4g.disconnect()
            bp4g.disconnectDevice()
        }
        log.T(text: deviceConnected ? "unConnect" : "disconnectDevice")
    }
    
    /// bp 4g command
    func readHistorys() {
        bp4g.readHistorys()
        log.T(text: "Reading Historys")
    }

    func clearAllHistorys() {
        bp4g.clearAllHistorys()
        log.T(text: "Clearing All Historys")
    }

    func disconnect() {
        bp4g.disconnect()
        log.T(text: "Disconnecting")
    }

    func readUserAndVersionData() {
        bp4g.readUserAndVersionData()
        log.T(text: "Reading User and Version Data")
    }

    func writeUserData() {
        let ID = getNumAndLetterID(length: 11)
        let age = getAge()
        appInfo.user.bpmUserID = ID;
        appInfo.user.age = age as NSNumber;
        bp4g.writeUserData(ID, age: age)
        log.T(text: "Writing User Data：\(ID) Age: \(age)")
    }

    func readLastData() {
        bp4g.readLastData()
        log.T(text: "Reading Last Data")
    }

    func clearLastData() {
        bp4g.clearLastData()
        log.T(text: "Clearing Last Data")
    }

    func readDeviceInfo() {
        bp4g.readDeviceInfo()
        log.T(text: "Reading Device Info")
    }

    func readDeviceTime() {
        bp4g.readDeviceTime()
        log.T(text: "Reading Device Time")
    }

    func syncTiming() {
        bp4g.syncTiming()
        log.T(text: "Sync Timing")
    }

    func readSerialNumber() {
        bp4g.readSerialNumber()
        log.T(text: "Reading Serial Number")
    }

    
}


