//
//  Handler.swift
//  MicroLifeDeviceSDK-Demo_Swift
//
//  Created by 吳憲有 on 2024/5/3.
//

import Foundation
import MicroLifeDeviceSDK
import SwiftUI
import CoreTransferable

// SDK
let SDK_code = ""
let SDK_key = ""
let SDK_secret = ""

let sdk = BLESDK.share()
let bp3g = MicroLife3GBP.shareWhithAuthorization(SDK_code, key: SDK_key, secret: SDK_secret)
let bp4g = MicroLife4GBP.shareWhithAuthorization(SDK_code, key: SDK_key, secret: SDK_secret)
let thermometer = MicroLifeTemperature.shareWhithAuthorization(SDK_code, key: SDK_key, secret: SDK_secret)
let spo2 = MicroLifeOxygen.shareWhithAuthorization(SDK_code, key: SDK_key, secret: SDK_secret)
let weighingScale = MicroLifeWeight.shareWhithAuthorization(SDK_code, key: SDK_key, secret: SDK_secret)
let bgm700 = MicroLifeBGM700.shareWhithAuthorization(SDK_code, key: SDK_key, secret: SDK_secret)
let bgm800 = MicroLifeBGM800.shareWhithAuthorization(SDK_code, key: SDK_key, secret: SDK_secret)
let watchBPOffice = MicroLifeWatchBPOffice.shareWhithAuthorization(SDK_code, key: SDK_key, secret: SDK_secret)
let watchBPM1 = MicroLifeWatchBPM1.shareWhithAuthorization(SDK_code, key: SDK_key, secret: SDK_secret)
let watchBPHome = MicroLifeWatchBPHome.shareWhithAuthorization(SDK_code, key: SDK_key, secret: SDK_secret)
let watchBPO3 = MicroLifeWatchBPO3.shareWhithAuthorization(SDK_code, key: SDK_key, secret: SDK_secret)
let peakFlowMeter = MicroLifePFM.shareWhithAuthorization(SDK_code, key: SDK_key, secret: SDK_secret)
let electrocardiography = MicroLifeECG.shareWhithAuthorization(SDK_code, key: SDK_key, secret: SDK_secret)

// Color
let backgroundColorType = (UITraitCollection.current.userInterfaceStyle == .light) ? ColorType.warm: ColorType.dark
let textColorType = (UITraitCollection.current.userInterfaceStyle == .light) ? ColorType.darkBlack: ColorType.bright

// AppInfo
let appInfo = AppInfo.shared

// Log
let log = AppInfo.shared.log

/// ShowViewType
enum ShowViewType {
    case menu
    case oneforall
    case support
    case bp3g
    case bp4g
    case thermometer
    case spo2
    case weighingScale
    case bgm700
    case bgm800
    case watchBPOffice
    case watchBPM1
    case watchBPHome
    case watchBPO3
    case peakFlowMeter
    case electrocardiography
}

@Observable
class AppInfo {
    
    /// 顯示選單頁面
    var showViewType: ShowViewType = .menu
    
    var user: MicroLifeUserInfo
    
    var log = Log()

    static let shared = AppInfo() // 创建单例实例
    
    private init() {
        user = MicroLifeUserInfo()
        user.bpmUserID = getNumAndLetterID(length: 11)
        user.pfmUserID = getNumAndLetterID(length: 11)
        user.weightUserID = getNumID(length: 11)
        user.age = getAge() as NSNumber
        user.height = 168
        user.weight = 70
        user.gender = .male
        user.resistance = 0
        user.roleType = .normal
    } // 私有初始化方法，防止外部创建实例
    
}

/// 通用功能
/// 英數混合ID
func getNumAndLetterID(length :Int) -> String {
    var userID = ""
    let letters = "QWERTYUIOPASDFGHJKLZXCVBNM"
    for i in 0..<length {
        if i < 9 {
            userID.append("\(Int.random(in: 0...9))")
        } else {
            let index = letters.index(letters.startIndex, offsetBy: Int.random(in: 0..<26))
            userID.append(letters[index])
        }
    }
    return userID
}

/// 全數字ID
func getNumID(length :Int) -> String {
    var userID = ""
    for _ in 0..<length {
        userID.append("\(Int.random(in: 0...9))")
    }
    return userID
}

func getAge() -> Int {
    return (Int.random(in: 18...98))
}

func writeUserData() {
    weighingScale.writeUserData(appInfo.user.weightUserID,
                                age: appInfo.user.age.intValue,
                                gender: appInfo.user.gender,
                                height: appInfo.user.height.intValue,
                                roleType: appInfo.user.roleType,
                                weight: appInfo.user.weight.floatValue,
                                resistance: appInfo.user.resistance.intValue)
    log.T(text: "writeUserData:\(appInfo.user.parseDictionary())")
}
