//
//  Log.swift
//  MicroLifeDeviceSDK-Demo_Swift
//
//  Created by 吳憲有 on 2024/5/3.
//

import Foundation
import SwiftUI
import CoreTransferable

@Observable
class Log {
        
    var logs: [String] = ["DEMO\n***"]
            
    var fileName: String = ""
    
    var filePath: String = ""
        
    var date: String {
        var dateStr = ""
        if dateStr.isEmpty {
            let dateFormatter = DateFormatter()
            dateFormatter.dateFormat = "yyyy/MM/dd HH:mm:ss.SSS"
            dateStr = dateFormatter.string(from: Date())
        }
        return dateStr
    }
    
    init() {
        if let cachesDirectory = FileManager.default.urls(for: .cachesDirectory, in: .userDomainMask).last {
            let dateFormatter = DateFormatter()
            dateFormatter.dateFormat = "yyyy-MM-dd"
            let fileDate = dateFormatter.string(from: Date())
            fileName = "[Log][\(fileDate)].text"
            filePath = cachesDirectory.appendingPathComponent(fileName).path
        }
    }
    
    func T(text: String) {
        DispatchQueue.main.async {
            let log = "\(self.date)\n\(text)\n***"
            self.writeData(log)
        }
    }
    
    func V(vualue: Any) {
        DispatchQueue.main.async {
            if let data = vualue as? Data {
                let string = "length = \(data.count), bytes = \(data.debugDescription.uppercased())"
                let log = "\(self.date) \(string)\n"
                self.writeData(log)
            }
        }
    }
    
    func M(CMD: Int, model: Any) {
        DispatchQueue.main.async {
            if let dictionary = model as? NSDictionary {
                let log = "\(self.date)\nCMD：\(String(format: "%02lX", CMD))\nReadData Dictionary：\(dictionary)\n***"
                self.writeData(log)
                for key in dictionary.allKeys {
                    if let value = dictionary[key] {
                        self.M(CMD: CMD, model: value)
                    }
                }
            }
            if let value = model as? MicroLifeDataModel {
                let log = "\(self.date)\nCMD：\(String(format: "%02lX", CMD))\nReadData Model：\(value.parseDictionary())\n***"
                self.writeData(log)
            }
            if let value = model as? NSData {
                let log = "\(self.date)\nCMD：\(String(format: "%02lX", CMD))\nReadData Value：\(value.description)\n***"
                self.writeData(log)
            }
        }
    }
    
    func E(error: Error) {
        if error == nil {
            return
        }
        DispatchQueue.main.async {
            let log = "\(self.date)\nError:\(error)\n***"
            self.writeData(log)
        }
    }
    
    /// 清除
    func clear() {
        DispatchQueue.main.async {
            self.logs.removeAll()
            self.writeData("DEMO\n***")
        }
    }

    /// 寫入log文件中
    func writeData(_ stringData: Any) {
        print("\(stringData)")
        var writeStr = ""
        if let data = stringData as? Data {
            let string = "length = \(data.count), bytes = \(data.debugDescription.uppercased())\n"
            logs.append(string)
            writeStr += string
        } else if let string = stringData as? String {
            logs.append(string)
            writeStr += "\(string)"
        }
        if let writeData = writeStr.data(using: .utf8) {
            let fileManager = FileManager.default
            if !fileManager.fileExists(atPath: (filePath as NSString).deletingLastPathComponent, isDirectory: nil) {
                try? fileManager.createDirectory(atPath: (filePath as NSString).deletingLastPathComponent, withIntermediateDirectories: true, attributes: nil)
            }
            if !fileManager.fileExists(atPath: filePath) {
                try? writeData.write(to: URL(fileURLWithPath: filePath), options: .atomic)
            } else {
                if let fileHandler = FileHandle(forUpdatingAtPath: filePath) {
                    fileHandler.seekToEndOfFile()
                    fileHandler.write(writeData)
                    fileHandler.closeFile()
                }
            }
        }
    }
}

// 定义一个自定义类型，表示文件
struct LogText: Transferable {
    
    var fileURL: URL
    
    // 实现transferRepresentation属性，指定文件的表示方式
    static var transferRepresentation: some TransferRepresentation {
        FileRepresentation(exportedContentType: .text) { file in
            // 处理文件的生成和传输逻辑
            SentTransferredFile(file.fileURL)
        }
    }
    
}

struct LogZip: Transferable {
    
    var fileURL: URL
    
    // 实现transferRepresentation属性，指定文件的表示方式
    static var transferRepresentation: some TransferRepresentation {
        FileRepresentation(exportedContentType: .zip) { file in
            // 处理文件的生成和传输逻辑
            SentTransferredFile(file.fileURL)
        }
    }

}
