//
//  MenuInfo.swift
//  MicroLifeDeviceSDK-Demo_Swift
//
//  Created by 吳憲有 on 2024/1/18.
//

import Foundation
import SwiftUI
import MicroLifeDeviceSDK

@Observable
class MenuInfo: Identifiable {
    var id: UUID = UUID()
    var deviceType: Int!
    var title: String!
    var backgroundColor = Color.white
    var textColor = Color.black
    var device: MicroLifeDevice?
    
    init() {
        removalInfo()
    }

    func removalInfo() {
        // 依據日夜模式，設定背景顏色
        backgroundColor = Color.RainbowColor(type: backgroundColorType, alpha: 0.9)
        textColor = Color.RainbowColor(type: textColorType, alpha: 1.0)
        device = nil
    }
    
    func addDevice(_ device: MicroLifeDevice) {
        self.device = device
    }
}

class MenuProvider {
    
    static var menus = MenuProvider.getMenus()
    
    static func getMenus() -> [MenuInfo] {
        var menus: [MenuInfo] = []
        
        let menuInfo1 = MenuInfo()
        menuInfo1.title = "Scan Device"
        menuInfo1.deviceType = 100
//        menus.append(menuInfo1)
        
        let menuInfo2 = MenuInfo()
        menuInfo2.title = "Cloud API"
        menuInfo2.deviceType = 101
//        menus.append(menuInfo2)
        
        let menuInfo3 = MenuInfo()
        menuInfo3.title = "SEND SUPPORT EMAIL"
        menuInfo3.deviceType = 102
        menus.append(menuInfo3)
        
        let menuInfo4 = MenuInfo()
        menuInfo4.title = "ONE FOR ALL"
        menuInfo4.deviceType = 103
        menus.append(menuInfo4)
        
        let menuInfo5 = MenuInfo()
        menuInfo5.title = "3G BP"
        menuInfo5.deviceType = DeviceType.type3GBP.rawValue
        menus.append(menuInfo5)
        
        let menuInfo6 = MenuInfo()
        menuInfo6.title = "4G BP"
        menuInfo6.deviceType = DeviceType.type4GBP.rawValue
        menus.append(menuInfo6)
        
        let menuInfo7 = MenuInfo()
        menuInfo7.title = "Weight"
        menuInfo7.deviceType = DeviceType.typeWeight.rawValue
        menus.append(menuInfo7)
        
        let menuInfo8 = MenuInfo()
        menuInfo8.title = "Temperature"
        menuInfo8.deviceType = DeviceType.typeTemperature.rawValue
        menus.append(menuInfo8)
        
        let menuInfo9 = MenuInfo()
        menuInfo9.title = "Oxygen"
        menuInfo9.deviceType = DeviceType.typeOxygen.rawValue
        menus.append(menuInfo9)
        
        let menuInfo10 = MenuInfo()
        menuInfo10.title = "WatchBP Home"
        menuInfo10.deviceType = DeviceType.typeWatchBPHome.rawValue
        menus.append(menuInfo10)
        
        let menuInfo11 = MenuInfo()
        menuInfo11.title = "WatchBP O3"
        menuInfo11.deviceType = DeviceType.typeWatchBPO3.rawValue
        menus.append(menuInfo11)
        
        let menuInfo12 = MenuInfo()
        menuInfo12.title = "WatchBP Office"
        menuInfo12.deviceType = DeviceType.typeWatchBPOffice.rawValue
        menus.append(menuInfo12)
        
        let menuInfo13 = MenuInfo()
        menuInfo13.title = "WatchBP M1"
        menuInfo13.deviceType = DeviceType.typeWatchBPM1.rawValue
        menus.append(menuInfo13)
        
        let menuInfo14 = MenuInfo()
        menuInfo14.title = "Blood Sugar (BGM700)"
        menuInfo14.deviceType = DeviceType.typeBGM700.rawValue
        menus.append(menuInfo14)
        
        let menuInfo15 = MenuInfo()
        menuInfo15.title = "Blood Sugar (BGM800)"
        menuInfo15.deviceType = DeviceType.typeBGM800.rawValue
        menus.append(menuInfo15)
        
        let menuInfo16 = MenuInfo()
        menuInfo16.title = "Peakflow Meter"
        menuInfo16.deviceType = DeviceType.typePFM.rawValue
//        menus.append(menuInfo16)
        
        let menuInfo17 = MenuInfo()
        menuInfo17.title = "Electrocardiography"
        menuInfo17.deviceType = DeviceType.typeECG.rawValue
//        menus.append(menuInfo17)
        
        let menuInfo18 = MenuInfo()
        menuInfo18.title = "PPG"
        menuInfo18.deviceType = DeviceType.typePPG.rawValue
//        menus.append(menuInfo18)
        
        return menus
    }

    
    static func refreshData() {
        for menu in menus {
            menu.removalInfo()
        }
    }
}
