//
//  MenuViewModel.swift
//  MicroLifeDeviceSDK-Demo_Swift
//
//  Created by 吳憲有 on 2024/1/18.
//

import Foundation

@Observable
class MenuViewModel {

    var menus: [MenuInfo] = MenuProvider.menus
        
    var state: CBManagerState = CBManagerState.unknown
        
    func setupBLESDK() {
        // Implement BLESDK setup logic
        
        sdk.cancelAllConnect()
        
        let autoConnect = false
        bp3g.autoConnect(autoConnect)
        bp4g.autoConnect(autoConnect)
        thermometer.autoConnect(autoConnect)
        spo2.autoConnect(autoConnect)
        weighingScale.autoConnect(autoConnect)
        bgm700.autoConnect(autoConnect)
        bgm800.autoConnect(autoConnect)
        watchBPOffice.autoConnect(autoConnect)
        watchBPM1.autoConnect(autoConnect)
        watchBPHome.autoConnect(autoConnect)
        watchBPO3.autoConnect(autoConnect)
        peakFlowMeter.autoConnect(autoConnect)
        electrocardiography.autoConnect(autoConnect)

        // 將裝置添加到 SDK 中並開始掃描
        sdk.device([bp3g, bp4g, thermometer, spo2, weighingScale, bgm700, bgm800, watchBPOffice, watchBPM1, watchBPHome, watchBPO3, peakFlowMeter, electrocardiography])

        sdk.getDidUpdateStateBlock { state in
            log.T(text:"isBLEStatePoweredOn: \(state)")
            self.state = state
        }
        
        sdk.getScanDeviceBlock { checkMacType, device in
            if let microLifeDevice = device as? MicroLifeDevice {
                self.addDevice(microLifeDevice)
            }
        } cancelScanBlock: {
            MenuProvider.refreshData()
        }
    }
    
    func refreshData() {
        sdk.startScan()
    }
    
    func cancelScan() {
        sdk.cancelScan()
    }

    func addDevice(_ device: MicroLifeDevice) {
        for menu in menus {
            if menu.deviceType == device.deviceType.rawValue {
                menu.device = device
                return
            }
        }
    }
}
