//
//  OFAViewModel.swift
//  MicroLifeDeviceSDK-Demo_Swift
//
//  Created by 吳憲有 on 2024/5/3.
//

import Foundation

@Observable
class OFAViewModel {
    func setupBLESDK() {
        
        log.clear()
        
        sdk.cancelAllConnect()
        
        let autoConnect = true
        bp3g.autoConnect(autoConnect)
        bp4g.autoConnect(autoConnect)
        thermometer.autoConnect(autoConnect)
        spo2.autoConnect(autoConnect)
        weighingScale.autoConnect(autoConnect)
        bgm700.autoConnect(autoConnect)
        bgm800.autoConnect(autoConnect)
        watchBPOffice.autoConnect(autoConnect)
        watchBPM1.autoConnect(autoConnect)
        watchBPHome.autoConnect(autoConnect)
        watchBPO3.autoConnect(autoConnect)
        peakFlowMeter.autoConnect(autoConnect)
        electrocardiography.autoConnect(autoConnect)

        /// 將裝置添加到 SDK 中並開始掃描
        let devces = [bp3g, bp4g, thermometer, spo2, weighingScale, bgm700, bgm800, watchBPOffice, watchBPM1, watchBPHome, watchBPO3, peakFlowMeter]
        sdk.device(devces)
        
        /// 掃描資訊
        sdk.getScanDeviceBlock { checkMacType, device in
            if let microLifeDevice = device as? MicroLifeDevice {
                log.T(text: "ScanDevice:\nName:\(microLifeDevice.name) MacType:\(checkMacType.description) RSSI:\(microLifeDevice.rssi)\nUUID:\(microLifeDevice.uuid)")
            }
        } cancelScanBlock: {
            log.T(text: "Cancel Scan")
        }
        
        /// 連線資訊
        sdk.getConnectDeviceStateBlock { device, _, _, state, error in
            log.T(text: "Device: \(device)\nPeripheral State: \(state)")
        } cancelAllDevicesConnectionBlock: {
            log.T(text: "Cancel All Devices Connection")
            sdk.startScan()
        }
        
        /// bp3g
        bp3g.getReadDataModelBlock { CMD, model, error in
            log.M(CMD: CMD, model: model)
            log.E(error: error)
            guard let command = CMD3G(rawValue: CMD) else {
                // Handle the case when CMD does not match any enumeration value
                // For example, log an error or perform other error-handling tasks
                return
            }

            switch command {
            case .readUserAndVersionData:
                bp3g.readHistorys()
            // Handle reading user and version data
            case .readHistorys:
                bp3g.disconnect()
            // Handle reading history data
            default:
                // Handle other cases if needed
                // For example, log an error or perform other error-handling tasks
                break
            }
        }
        
        /// bp4g
        bp4g.getReadDataModelBlock { CMD, model, error in
            log.M(CMD: CMD, model: model)
            log.E(error: error)
            switch CMD {
            case CMD4G.readUserAndVersionData.rawValue:
                // Execute the readHistorys method for bp4g
                bp4g.readHistorys()
            case CMD4G.readHistorys.rawValue:
                // Execute the disconnect method for bp4g
                bp4g.disconnect()
            default:
                // Handle other cases if needed
                break
            }
        }
        
        bp4g.getPairingErrorBlock { error in
            log.T(text: "Pairing Error:\(error)")
        }
        
        /// temperature
        thermometer.getReadDataModelBlock { CMD, model, error in
            log.M(CMD: CMD, model: model)
            log.E(error: error)
            switch CMD {
            default:
                // Handle other cases if needed
                break
            }
        }
        
        /// oxygen
        spo2.getReadDataModelBlock { CMD, model, error in
            log.M(CMD: CMD, model: model)
            log.E(error: error)
            switch CMD {
            default:
                // Handle other cases if needed
                break
            }
        }
        
        /// weight
        weighingScale.deviceConnectedBlock {
            writeUserData()
        }
        weighingScale.getReadDataModelBlock { CMD, model, error in
            log.M(CMD: CMD, model: model)
            log.E(error: error)
            switch CMD {
            case CMDWeight.wakeUpScale.rawValue:
                // Log and handle waking up the scale
                log.T(text: "WakeUp Scale")
                weighingScale.readHistorys()
            case CMDWeight.sleepScale.rawValue:
                // Log and handle putting the scale to sleep
                log.T(text: "Sleep Scale")
                weighingScale.disconnect()
            case CMDWeight.measurementResult.rawValue:
                // Log and handle measurement result
                log.T(text: "Measurement Result")
            case CMDWeight.readHistorys.rawValue:
                // Log and handle reading history data
                log.T(text: "Read Historys")
            case CMDWeight.updateUserInfo.rawValue:
                // Log and handle updating user info
                log.T(text: "Update User Info")
            case CMDWeight.clearAllUserInfo.rawValue:
                // Log and handle clearing all user info
                log.T(text: "Clear All User Info")
            case CMDWeight.noMoreOfflineData.rawValue:
                // Log and handle indication of no more offline data
                log.T(text: "No More Offline Data")
            case CMDWeight.lowPower.rawValue:
                // Log and handle low power indication
                log.T(text: "Low Power")
            case CMDWeight.syncSystemClock.rawValue:
                // Log and handle synchronizing system clock
                log.T(text: "Sync System Clock")
            default:
                // Handle other cases if needed
                break
            }
        }
        
        /// BloodSugar
        bgm700.getReadDataModelBlock { CMD, model, error in
            log.M(CMD: CMD, model: model)
            log.E(error: error)
            switch CMD {
            default:
                // Handle other cases if needed
                break
            }
        }
        
        bgm800.getReadDataModelBlock { CMD, model, error in
            log.M(CMD: CMD, model: model)
            log.E(error: error)
            switch CMD {
            default:
                // Handle other cases if needed
                break
            }
        }
        
        /// watchBPM1
        watchBPM1.getReadDataModelBlock { CMD, model, error in
            log.M(CMD: CMD, model: model)
            log.E(error: error)
            switch CMD {
            case CMDWatchBPM1.readHistorys.rawValue:
                // Disconnect the watchBPM1 device
                watchBPM1.disconnect()
            case CMDWatchBPM1.readUserAndVersionData.rawValue:
                // Read all history data from watchBPM1
                watchBPM1.readAllHistorys()
            default:
                // Handle other cases if needed
                break
            }
        }
        
        /// watchBPHome
        watchBPHome.getReadDataModelBlock { CMD, model, error in
            log.M(CMD: CMD, model: model)
            log.E(error: error)
            switch CMD {
            case CMDWatchBPHome.readUsualModeHistoryDataIncludeEachMeasurement.rawValue:
                // Disconnect the watchBPHome device
                watchBPHome.disconnect()
            case CMDWatchBPHome.readUserAndVersionData.rawValue:
                // Read user and version data from watchBPHome
                watchBPHome.readUserAndVersionData()
            default:
                // Handle other cases if needed
                break
            }
        }
        
        /// watchBPO3
        watchBPO3.getReadDataModelBlock { CMD, model, error in
            log.M(CMD: CMD, model: model)
            log.E(error: error)
            switch CMD {
            case CMDWatchBPO3.bpo3ReadHistorys.rawValue:
                // Disconnect the watchBPO3 device
                watchBPO3.disconnect()
            case CMDWatchBPO3.bpo3ReadUserAndVersionData.rawValue:
                // Read all history data from watchBPO3
                watchBPO3.readAllHistorys()
            default:
                // Handle other cases if needed
                break
            }
        }
        
        /// peakFlowMeter
        peakFlowMeter.getReadDataModelBlock { CMD, model, error in
            log.M(CMD: CMD, model: model)
            log.E(error: error)
            switch CMD {
            case CMDPFM.readHistorys.rawValue:
                // Disconnect the peakFlowMeter device
                peakFlowMeter.disconnect()
            case CMDPFM.readLastData.rawValue:
                // Read history data from peakFlowMeter
                peakFlowMeter.readHistorys()
            case CMDPFM.readUserAndVersionData.rawValue:
                // Check the mode of peakFlowMeter
                peakFlowMeter.checkMode()
            case CMDPFM.readBestValue.rawValue:
                // Read the last data from peakFlowMeter
                peakFlowMeter.readLastData()
            case CMDPFM.checkMode.rawValue:
                // Read the best value from peakFlowMeter
                peakFlowMeter.readBestValue()
            default:
                // Handle other cases if needed
                break
            }
        }
        
        peakFlowMeter.getPairingErrorBlock { error in
            log.T(text: "Pairing Error:\(error)")
        }
    }

    /// The method to write user data to the weighing scale
    func refreshData() {
        log.clear()
        sdk.cancelAllConnect()
        sdk.startScan()
    }
}
