//
//  OxygenView.swift
//  MicroLifeDeviceSDK-Demo_Swift
//
//  Created by 吳憲有 on 2024/5/13.
//

import SwiftUI

struct OxygenView: View {
    
    @Environment(AppInfo.self) var appInfo
    
    @Environment(\.horizontalSizeClass) var horizontalSizeClass
    
    @Environment(\.verticalSizeClass) var verticalSizeClass
    
    var viewModel = OxygenViewModel()
    
    var menuInfo: MenuInfo
    
    var body: some View {
        let orientation = DeviceOrientation.checkOrientation(horizontalSizeClass: horizontalSizeClass, verticalSizeClass: verticalSizeClass)
        ZStack {
            switch orientation {
            case .iPhoneHorizontal, .OtherDevice:
                HorizontalLayoutView(scale: 0.85) {
                    viewModel.startScan()
                } cancelScanAction: {
                    viewModel.cancelScan()
                } disconnectDeviceAction: {
                    viewModel.disconnectDevice()
                } commandViewContent: {
                    Spacer()
                }
            case .iPhoneVertical, .iPad:
                VerticalLayoutView(scale: 0.75) {
                    viewModel.startScan()
                } cancelScanAction: {
                    viewModel.cancelScan()
                } disconnectDeviceAction: {
                    viewModel.disconnectDevice()
                } commandViewContent: {
                    Spacer()
                }
            }
        }
        .toolbar {
            CustomToolbarItem(
                filePath: log.filePath,
                fileName: log.fileName,
                refreshAction: {
                    viewModel.refreshData()
                })
        }
        .navigationTitle(menuInfo.title)
        .padding(.horizontal, 8)
        .onChange(of: appInfo.showViewType) {
            if appInfo.showViewType == .spo2 {
                viewModel.setupBLESDK()
            }
        }
        .onAppear {
            appInfo.showViewType = .spo2
            viewModel.setupBLESDK()
        }
        .onDisappear {
            appInfo.showViewType = .menu
            viewModel.disconnectDevice()
        }
    }
}

#Preview {
    OxygenView(menuInfo: MenuInfo())
}
