//
//  OxygenViewModel.swift
//  MicroLifeDeviceSDK-Demo_Swift
//
//  Created by 吳憲有 on 2024/5/13.
//

import Foundation
import SwiftUI
import MicroLifeDeviceSDK

@Observable
class OxygenViewModel {

    var deviceConnected = spo2.deviceConnected
    
    func setupBLESDK() {
        
        log.clear()

        sdk.cancelAllConnect()
        
        spo2.autoConnect(true)

        spo2.getDidUpdateStateBlock { state in
            log.T(text:"isBLEStatePoweredOn: \(state)")
        } scanDeviceBlock: { checkMacType, device in
            if let microLifeDevice = device as? MicroLifeDevice {
                log.T(text: "ScanDevice:\nName:\(microLifeDevice.name) MacType:\(checkMacType.description) RSSI:\(microLifeDevice.rssi)\nUUID:\(microLifeDevice.uuid)")
            }
        } cancelScanBlock: {
            log.T(text:"Cancel Scan")
        } connectDeviceStateBlock: { device, central, peripheral, state, error in
            log.T(text:"Device: \(device)\nPeripheral State: \(state)")
            log.E(error: error)
        } cancelAllDevicesConnectionBlock: {
            log.T(text:"Cancel All Devices Connection")
            spo2.startScan()
        }
        
        spo2.deviceConnectedBlock {
            self.deviceConnected = spo2.deviceConnected
        }
        
        spo2.getReadDataModelBlock { CMD, model, error in
            log.M(CMD: CMD, model: model)
            log.E(error: error)
            guard let command = CMDOxygen(rawValue: CMD) else {
                return
            }
            switch command {
            case .readAlarmInfo:
                // Handle reading alarm info
                break
            case .saturationAndPulseRateData:
                // Handle saturation and pulse rate data
                break
            case .saturationAndPulseRateAlarmLimits:
                // Handle saturation and pulse rate alarm limits
                break
            default:
                // Handle other cases if needed
                break
            }
        }
        
        spo2.getValidationErrorBlock { item, info, value in
            log.T(text: "\(item)\ninfo:\(info)\nvalue:\(value)")
        }
    }
    
    func refreshData() {
        log.clear()
        log.T(text:"refreshData")
        disconnectDevice()
        startScan()
    }
    
    func startScan() {
        spo2.startScan()
        log.T(text: "Start Scan")
    }

    func cancelScan() {
        spo2.cancelScan()
        log.T(text: "Cancel Scan")
    }
    
    func disconnectDevice() {
        if deviceConnected {
            spo2.disconnectDevice()
        }
        log.T(text: deviceConnected ? "unConnect" : "disconnectDevice")
    }
}
