//
//  BLEDeviceInfo.h
//  MicroLifeDeviceSDK
// Bluetooth device information
//  Created by willy.wu on 2020/11/19.
//

#import <Foundation/Foundation.h>
#import <MicroLifeDeviceSDK/BLESDK.h>

NS_ASSUME_NONNULL_BEGIN

@interface BLEDeviceInfo : NSObject

@property (nonatomic, strong) BLESDK *sdk;

/// name
@property (nonatomic, strong) NSString *name;

/// UUID
@property (nonatomic, strong) NSString *UUID;

/// Mac Address
@property (nonatomic, strong) NSData *mac;

/// RSSI
@property (nonatomic, strong) NSNumber *RSSI;

/// Peripheral
@property (nonatomic, strong) CBPeripheral *currPeripheral;

/// noty characteristic
@property (nonatomic, strong) CBCharacteristic *characteristicRx;

/// GATT characteristic
@property (nonatomic, strong) NSMutableArray *characteristicGx;

/// write characteristic
@property (nonatomic, strong) CBCharacteristic *characteristicTx;

/// DvertisementData
@property (nonatomic, strong) NSDictionary *dvertisementData;


/// init
/// @param baby baby
/// @param peripheral peripheral
/// @param dvertisementData dvertisementData
/// @param RSSI RSSI
+ (instancetype)initWithDiscoverBaby:(id)baby Peripheral:(CBPeripheral *)peripheral DvertisementData:(NSDictionary *)dvertisementData RSSI:(NSNumber *)RSSI;

/// check Mac Type
/// 0: Mac None
/// 1: Mac format is correct
/// 2: Mac format error
- (NSInteger)checkMacType;

/// Connect
- (void)connectDevice;

/// Disconnect
- (void)disconnectDevice;

/// Set Auto Reconnect
- (void)setAutoReconnect;

/// Cancel Auto Reconnect
- (void)cancelAutoReconnect;

- (NSMutableDictionary *)parseDictionary;

@end

NS_ASSUME_NONNULL_END
