//
//  BLESDK.h
//  MicroLifeDeviceSDK
// 1. Scan and connect with Bluetooth device.
// 2. The command is sent.
// 3. Reply to receive.
//  Created by willy.wu on 2020/11/18.
//

#import <Foundation/Foundation.h>
#import <CoreBluetooth/CoreBluetooth.h>

NS_ASSUME_NONNULL_BEGIN

typedef void(^didUpdateStateBlock) (CBManagerState state);

typedef void(^scanDeviceBlock) (NSInteger checkMacType, id device);
typedef void(^cancelScanBlock) (void);
typedef void(^connectDeviceStateBlock) (id device, CBCentralManager *central, CBPeripheral *peripheral, CBPeripheralState state, NSError * error);
typedef void(^cancelAllDevicesConnectionBlock) (void);

typedef void(^discoverServicesBlock) (id device, CBPeripheral *peripheral, NSError * error);
typedef void(^discoverCharacteristicsBlock) (id device, CBPeripheral *peripheral, CBService *service, NSError * error);
typedef void(^discoverDescriptorsForCharacteristicBlock) (id device, CBPeripheral *peripheral, CBCharacteristic *characteristics, NSError * error);

typedef void(^updateNotificationStateForCharacteristicBlock) (CBCharacteristic *characteristic, NSError * error);
typedef void(^readRSSIBlock) (NSNumber *RSSI, NSError * error);

typedef void(^writeValueBlock) (CBCharacteristic *characteristic, NSError * error);
typedef void(^readValueBlock) (id device, CBPeripheral *peripheral, CBCharacteristic *characteristics,  NSError * error);
typedef void(^readValueForDescriptorsBlock) (id device, CBPeripheral *peripheral, CBDescriptor *descriptor, NSError * error);
typedef void(^readNotifyValueForCharacteristicBlock) (id device, CBPeripheral *peripheral, CBCharacteristic *characteristics, NSError * error);
@interface BLESDK : NSObject

+ (instancetype)shareOne;

+ (instancetype)share;

/// show Log
/// @param showLog showLog
- (void)showLog:(BOOL)showLog;

/// Set up Bluetooth
/// @param bluetooth MicroLifeDevice Class
- (void)device:(NSArray *)bluetooth;

/// Set Sacn stop Time
/// @param time time
- (void)stopTime:(NSInteger)time;

/// Set RSSI
/// @param rssi RSSI
- (void)rssi:(NSInteger)rssi;

/// When CBManagerStatePoweredOn automatically opens scanning [Default: Open]
/// @param autoScan autoSca
- (void)autoScan:(BOOL)autoScan;

/// Start Scan
- (void)startScan;

/// Cancel Scan
- (void)cancelScan;

/// Cancel All Connect
- (void)cancelAllConnect;

/// Get the currently connected devices
- (NSArray *)findConnectedDevices;

#pragma mark -
/// Connect
/// @param device device
- (void)connectDevice:(id)device;

/// Disconnect
/// @param device device
- (void)disconnectDevice:(id)device;

/// Set AutoReconnect
/// @param device device
- (void)setAutoReconnect:(id)device;

/// Cancel AutoReconnect
/// @param device device
- (void)cancelAutoReconnect:(id)device;

/// set Notifiy
/// @param device device
- (void)setNotifiy:(id)device characteristic:(CBCharacteristic *)characteristic;

/// write Value
/// @param value value
/// @param deviceInfo BLEDeviceInfo
- (BOOL)writeValue:(NSString *)value DeviceInfo:(id)deviceInfo;

/// HEARTCHECK
/// @param value value
/// @param deviceInfo BLEDeviceInfo
- (void)HEARTCHECK:(NSString *)value DeviceInfo:(id)deviceInfo;

///  When CentralManager state changed
/// @param block didUpdateStateBlock
- (void)getDidUpdateStateBlock:(didUpdateStateBlock)block;

/// Scan Device
/// @param scanDeviceBlock scanDeviceBlock
/// @param cancelScanBlock cancelScanBlock
- (void)getScanDeviceBlock:(scanDeviceBlock)scanDeviceBlock CancelScanBlock:(cancelScanBlock)cancelScanBlock;

/// Connect Device State
/// @param connectDeviceStateBlock connectDeviceStateBlock
/// @param cancelAllDevicesConnectionBlock cancelAllDevicesConnectionBlock
- (void)getConnectDeviceStateBlock:(connectDeviceStateBlock)connectDeviceStateBlock CancelAllDevicesConnectionBlock:(cancelAllDevicesConnectionBlock)cancelAllDevicesConnectionBlock;

/// Discover Services
/// @param block discoverServicesBlock
- (void)getDiscoverServicesBlock:(discoverServicesBlock)block;

/// Discover Characteristics
/// @param block discoverCharacteristicsBlock
- (void)getDiscoverCharacteristicsBlock:(discoverCharacteristicsBlock)block;

/// Discover Descriptors For Characteristic
/// @param block discoverDescriptorsForCharacteristicBlock
- (void)getDiscoverDescriptorsForCharacteristicBlock:(discoverDescriptorsForCharacteristicBlock)block;

/// Update Notification State For Characteristic
/// @param block updateNotificationStateForCharacteristicBlock
- (void)getUpdateNotificationStateForCharacteristicBlock:(updateNotificationStateForCharacteristicBlock)block;

/// Read RSSI
/// @param block readRSSIBlock
- (void)getReadRSSIBlock:(readRSSIBlock)block;

/// Write Value For Characteristic
/// @param block writeValueBlock
- (void)getWriteValueForCharacteristicBlock:(writeValueBlock)block;

/// Read Value For Characteristic
/// @param block readValueBlock
- (void)getReadValueForCharacteristicBlock:(readValueBlock)block;

/// ReadValue For Descriptors
/// @param block readValueForDescriptorsBlock
- (void)getReadValueForDescriptorsBlock:(readValueForDescriptorsBlock)block;

/// Read Notify Value For Characteristic
/// @param block readNotifyValueForCharacteristicBlock
- (void)getReadNotifyValueForCharacteristicBlock:(readNotifyValueForCharacteristicBlock)block;

@end

NS_ASSUME_NONNULL_END
