//
//  SupportView.swift
//  MicroLifeDeviceSDK-Demo_Swift
//
//  Created by 吳憲有 on 2024/5/8.
//

import SwiftUI
import MessageUI

class MailDelegate: NSObject, MFMailComposeViewControllerDelegate {
    func mailComposeController(_ controller: MFMailComposeViewController, didFinishWith result: MFMailComposeResult, error: Error?) {
        controller.dismiss(animated: true)
    }
}

struct SupportView: View {
    
    @Environment(AppInfo.self) var appInfo
        
    let backgroundColor = Color.RainbowColor(type: backgroundColorType, alpha: 0.3)

    let logManager = LogManager.sharedInstance()
    
    let mailDelegate = MailDelegate()

    var menuInfo: MenuInfo
    
    @State var logPath = ""

    @State var paths: [String] = []
        
    @State var user = ""

    @State var emailMessageBody = ""
    
    var body: some View {
        VStack {
            if paths.count > 0 {
                Text("log file:\n\(paths.debugDescription)")
                    .multilineTextAlignment(.leading)
                    .frame(maxWidth: .infinity, alignment: .topLeading)
                    .containerRelativeFrame(.vertical) { height, _ in height * 0.3 }
            }
            
            /// 邮件分享 內容輸入匡
            TextField("user", text: $user)
                .containerRelativeFrame(.vertical) { height, _ in height * 0.1 }
                .background(backgroundColor)

            TextField("message", text: $emailMessageBody)
                .containerRelativeFrame(.vertical) { height, _ in height * 0.3 }
                .background(backgroundColor)
            
            if FileManager.default.fileExists(atPath: logPath) {
                HStack {
                    ShareLink(item: LogZip(fileURL: URL(fileURLWithPath: logPath)),
                              preview: SharePreview(menuInfo.title))
                    if MFMailComposeViewController.canSendMail() && user.count > 0 && emailMessageBody.count > 0 {
                        Spacer()
                        Button("Email") {
                            let appName = Bundle.main.infoDictionary?["CFBundleName"] as? String
                            sendEmailWithDynamicTitleAndContent(
                                zipFilePath: logPath,
                                appName: appName ?? "MicroLifeDeviceSDK",
                                appVersion: MicroLifeDeviceSDKVersionNumber.description,
                                user: user,
                                emailMessage: emailMessageBody)
                        }
                    }
                }
                .padding(.horizontal, 50)
                Spacer()
            } else {
                Text("No log file")
            }
        }
        .navigationTitle(menuInfo.title)
        .padding(.horizontal, 8)
        .onAppear {
            appInfo.showViewType = .support
            logManager.getLogZip { success, logZipPath in
                if !success {
                    return
                }
                logPath = logZipPath
            }
            let logPaths: [Any] = logManager.getLogFilePaths()
            if let paths = logPaths as? [String] {
                self.paths = paths
            }
        }
        .onDisappear {
            appInfo.showViewType = .menu
        }
    }
    
    func sendEmailWithDynamicTitleAndContent(zipFilePath: String, appName: String, appVersion: String, user: String, emailMessage: String) {
        let mailComposeViewController = MFMailComposeViewController()
        mailComposeViewController.setToRecipients(["Serviceios@Microlife.com.tw"])
        
        // 动态设置邮件标题
        let currentLocale = Locale.current
        let title = "\(appName) \(appVersion) | iOS | \(user) | \(currentLocale.identifier)"
        mailComposeViewController.setSubject(title)
        
        // 动态设置邮件内容，包括标题和内容
        let currentDevice = UIDevice.current
        
        let fullEmailContent = String(format: "\nSDK Version: %.1f\nDevice OS Version: %@\nDevice MODEL: %@\nLanguage: %@\nLocate: %@\nMessage: %@", MicroLifeDeviceSDKVersionNumber, currentDevice.systemVersion, currentDevice.model, currentLocale.identifier, currentLocale.identifier, emailMessage)
        mailComposeViewController.setMessageBody(fullEmailContent, isHTML: false)
        
        mailComposeViewController.mailComposeDelegate = mailDelegate // 需要实现 MFMailComposeViewControllerDelegate

        let zipFileURL = URL(fileURLWithPath: zipFilePath)
        if let zipData = try? Data(contentsOf: zipFileURL) {
            mailComposeViewController.addAttachmentData(zipData, mimeType: "application/zip", fileName: zipFileURL.lastPathComponent)
        }

        if let keyWindow = UIApplication.shared.connectedScenes.first as? UIWindowScene {
            if let rootViewController = keyWindow.windows.first?.rootViewController {
                rootViewController.present(mailComposeViewController, animated: true, completion: nil)
            }
        }
    }

}

#Preview {
    SupportView(menuInfo: MenuInfo())
}
