//
//  TemperatureViewModel.swift
//  MicroLifeDeviceSDK-Demo_Swift
//
//  Created by 吳憲有 on 2024/5/10.
//

import Foundation

@Observable
class TemperatureViewModel {

    var deviceConnected = thermometer.deviceConnected
    
    func setupBLESDK() {
        
        log.clear()

        sdk.cancelAllConnect()
        
        thermometer.autoConnect(true)

        thermometer.getDidUpdateStateBlock { state in
            log.T(text:"isBLEStatePoweredOn: \(state)")
        } scanDeviceBlock: { checkMacType, device in
            if let microLifeDevice = device as? MicroLifeDevice {
                log.T(text: "ScanDevice:\nName:\(microLifeDevice.name) MacType:\(checkMacType.description) RSSI:\(microLifeDevice.rssi)\nUUID:\(microLifeDevice.uuid)")
            }
        } cancelScanBlock: {
            log.T(text:"Cancel Scan")
        } connectDeviceStateBlock: { device, central, peripheral, state, error in
            log.T(text:"Device: \(device)\nPeripheral State: \(state)")
            log.E(error: error)
        } cancelAllDevicesConnectionBlock: {
            log.T(text:"Cancel All Devices Connection")
            thermometer.startScan()
        }
        
        thermometer.deviceConnectedBlock {
            self.deviceConnected = thermometer.deviceConnected
        }
        
        thermometer.getReadDataModelBlock { CMD, model, error in
            log.M(CMD: CMD, model: model)
            log.E(error: error)
            guard let command = CMDTemperature(rawValue: CMD) else {
                return
            }
            switch command {
            case .readMeasureData:
                // Handle reading temperature measurement data
                break
            case .readDeviceInfo:
                // Handle reading device information
                break
            default:
                // Handle other cases if needed
                break
            }
        }
        
        thermometer.getValidationErrorBlock { item, info, value in
            log.T(text: "\(item)\ninfo:\(info)\nvalue:\(value)")
        }
    }
    
    func refreshData() {
        log.clear()
        log.T(text:"refreshData")
        disconnectDevice()
        startScan()
    }
    
    func startScan() {
        thermometer.startScan()
        log.T(text: "Start Scan")
    }

    func cancelScan() {
        thermometer.cancelScan()
        log.T(text: "Cancel Scan")
    }
    
    func disconnectDevice() {
        if deviceConnected {
            thermometer.disconnectDevice()
        }
        log.T(text: deviceConnected ? "unConnect" : "disconnectDevice")
    }
}
