//
//  WatchBPHomeView.swift
//  MicroLifeDeviceSDK-Demo_Swift
//
//  Created by 吳憲有 on 2024/5/17.
//

import SwiftUI

struct WatchBPHomeView: View {
    @Environment(AppInfo.self) var appInfo

    @Environment(\.horizontalSizeClass) var horizontalSizeClass

    @Environment(\.verticalSizeClass) var verticalSizeClass

    var viewModel = WatchBPHomeViewModel()

    var menuInfo: MenuInfo

    var body: some View {
        let orientation = DeviceOrientation.checkOrientation(horizontalSizeClass: horizontalSizeClass, verticalSizeClass: verticalSizeClass)
        ZStack {
            switch orientation {
                case .iPhoneHorizontal, .OtherDevice:
                    HorizontalLayoutView(scale: 0.6) {
                        viewModel.startScan()
                    } cancelScanAction: {
                        viewModel.cancelScan()
                    } disconnectDeviceAction: {
                        viewModel.disconnectDevice()
                    } commandViewContent: {
                        CustomWatchBPHomeView(viewModel: viewModel)
                    }

                case .iPhoneVertical, .iPad:
                    VerticalLayoutView(scale: 0.75) {
                        viewModel.startScan()
                    } cancelScanAction: {
                        viewModel.cancelScan()
                    } disconnectDeviceAction: {
                        viewModel.disconnectDevice()
                    } commandViewContent: {
                        CustomWatchBPHomeView(viewModel: viewModel)
                    }
            }
        }
        .toolbar {
            CustomToolbarItem(
                filePath: log.filePath,
                fileName: log.fileName,
                refreshAction: {
                    viewModel.refreshData()
                })
        }
        .navigationTitle(menuInfo.title)
        .padding(.horizontal, 8)
        .onChange(of: appInfo.showViewType) {
            if appInfo.showViewType == .watchBPHome {
                viewModel.setupBLESDK()
            }
        }
        .onAppear {
            appInfo.showViewType = .watchBPHome
            viewModel.setupBLESDK()
        }
        .onDisappear {
            appInfo.showViewType = .menu
            viewModel.disconnectDevice()
        }
    }
}

struct CustomWatchBPHomeView: View {
    var viewModel: WatchBPHomeViewModel

    var body: some View {
        VStack(alignment: .trailing, spacing: 16) {
            Button(action: {
                viewModel.readUsualModeHistoryData()
            }) {
                Text("Read Usual Mode History Data")
            }
            Button(action: {
                viewModel.readDiagnosticModeHistoryData()
            }) {
                Text("Read Diagnostic Mode History Data")
            }
            Button(action: {
                viewModel.clearHistoryData()
            }) {
                Text("Clear History Data")
            }
            Button(action: {
                viewModel.clearCurrentModeHistoryData()
            }) {
                Text("Clear Current Mode History Data")
            }
            Button(action: {
                viewModel.disconnect()
            }) {
                Text("Disconnect")
            }
            Button(action: {
                viewModel.writeDeviceTime()
            }) {
                Text("Write Device Time")
            }
            Button(action: {
                viewModel.writeUserID()
            }) {
                Text("Write User ID")
            }
            Button(action: {
                viewModel.readNocturnalModeSetting()
            }) {
                Text("Read Nocturnal Mode Setting")
            }
            Button(action: {
                viewModel.changeNocturnalModeSetting()
            }) {
                Text("Change Nocturnal Mode Setting")
            }
            Button(action: {
                viewModel.readDeviceIDAndInfo()
            }) {
                Text("Read Device ID and Info")
            }
            Button(action: {
                viewModel.readDeviceTime()
            }) {
                Text("Read Device Time")
            }
            Button(action: {
                viewModel.readUserAndVersionData()
            }) {
                Text("Read User and Version Data")
            }
            Button(action: {
                viewModel.readNocturnalModeHistoryData()
            }) {
                Text("Read Nocturnal Mode History Data")
            }
            Button(action: {
                viewModel.readSerialNumber()
            }) {
                Text("Read Serial Number")
            }
            Button(action: {
                viewModel.writeDeviceSettingOfMeasurement()
            }) {
                Text("Write Device Setting of Measurement")
            }
            Button(action: {
                viewModel.readDeviceSettingOfMeasurement()
            }) {
                Text("Read Device Setting of Measurement")
            }
            Button(action: {
                viewModel.readUsualModeHistoryDataIncludeEachMeasurement()
            }) {
                Text("Read Usual Mode History Data Including Each Measurement")
            }
        }
        .padding()
    }
}

#Preview {
    WatchBPHomeView(menuInfo: MenuInfo())
}
