//
//  WatchBPM1ViewModel.swift
//  MicroLifeDeviceSDK-Demo_Swift
//
//  Created by 吳憲有 on 2024/5/31.
//

import Foundation

@Observable
class WatchBPM1ViewModel {
    var deviceConnected = watchBPM1.deviceConnected

    var showSettings = false

    var settings = MicroLifeWatchBPSettingValues()

    func setupBLESDK() {
        log.clear()

        sdk.cancelAllConnect()

        watchBPM1.autoConnect(true)

        watchBPM1.getDidUpdateStateBlock { state in
            log.T(text: "isBLEStatePoweredOn: \(state)")
        } scanDeviceBlock: { checkMacType, device in
            if let microLifeDevice = device as? MicroLifeDevice {
                log.T(text: "ScanDevice:\nName:\(microLifeDevice.name) MacType:\(checkMacType.description) RSSI:\(microLifeDevice.rssi)\nUUID:\(microLifeDevice.uuid)")
            }
        } cancelScanBlock: {
            log.T(text: "Cancel Scan")
        } connectDeviceStateBlock: { device, _, _, state, error in
            log.T(text: "Device: \(device)\nPeripheral State: \(state)")
            log.E(error: error)
        } cancelAllDevicesConnectionBlock: {
            log.T(text: "Cancel All Devices Connection")
            self.settings = MicroLifeWatchBPSettingValues()
            watchBPM1.startScan()
        }

        watchBPM1.deviceConnectedBlock {
            self.deviceConnected = watchBPM1.deviceConnected
        }

        watchBPM1.getReadDataModelBlock { CMD, model, error in
            log.M(CMD: CMD, model: model)
            log.E(error: error)
            guard let command = CMDWatchBPM1(rawValue: CMD) else {
                return
            }
            switch command {
            case .readHistorys:
                // Handle reading history data...
                break
            case .clearAllHistorys:
                // Handle clearing all history data...
                break
            case .disconnect:
                // Handle disconnecting...
                break
            case .readUserAndVersionData:
                // Handle reading user and version data...
                break
            case .writeUser:
                // Handle writing user data...
                break
            case .readSettingValues:
                // Handle reading setting values...
                self.settings = model as! MicroLifeWatchBPSettingValues
            case .writeSettingValues:
                // Handle writing setting values...
                break
            case .readDeviceInfo:
                // Handle reading device info...
                break
            case .readDeviceTime:
                // Handle reading device time...
                break
            case .writeDeviceTime:
                // Handle writing device time...
                break
            case .readFunctionSettingValue:
                // Handle reading function setting value...
                break
            case .readBTModuleName:
                // Handle reading Bluetooth module name...
                break
            default:
                break
            }
        }

        watchBPM1.getValidationErrorBlock { item, info, value in
            log.T(text: "\(item)\ninfo:\(info)\nvalue:\(value)")
        }
    }

    func refreshData() {
        log.clear()
        log.T(text: "refreshData")
        disconnectDevice()
        startScan()
    }

    func startScan() {
        watchBPM1.startScan()
        log.T(text: "Start Scan")
    }

    func cancelScan() {
        watchBPM1.cancelScan()
        log.T(text: "Cancel Scan")
    }

    func disconnectDevice() {
        if deviceConnected {
            watchBPM1.disconnect()
            watchBPM1.disconnectDevice()
        }
        log.T(text: deviceConnected ? "unConnect" : "disconnectDevice")
    }

    /// WatchBPOffice command
    func readAllHistorys() {
        watchBPM1.readAllHistorys()
        log.T(text: "Reading all history data")
    }

    func clearAllHistorys() {
        watchBPM1.clearAllHistorys()
        log.T(text: "Clearing all history data")
    }

    func disconnect() {
        watchBPM1.disconnect()
        log.T(text: "Disconnecting from device")
    }

    func readUserAndVersionData() {
        watchBPM1.readUserAndVersionData()
        log.T(text: "Reading user and version data")
    }

    func writeUserID() {
        watchBPM1.writeUserID(appInfo.user.bpmUserID)
        log.T(text: "Writing user ID: \(appInfo.user.bpmUserID)")
    }

    func readSettingValues() {
        watchBPM1.readSettingValues()
        log.T(text: "Reading setting values")
    }

    func writeSettingValues() {
        watchBPM1.writeSettingValues(
            withSelectedAUS_HI_infPressure: settings.aus_HI_infPressure,
            hi_infPressure: settings.hi_infPressure,
            sw_AUTO_hide: settings.sw_AUS_Hide.boolValue,
            sw_SEL_silent: settings.sw_SEL_silent.boolValue,
            sw_AUS_Hide: settings.sw_AUS_Hide.boolValue,
            sw_AVG_no_include_first: settings.sw_AVG_no_include_first.boolValue,
            sw_CBP: settings.sw_CBP.boolValue,
            sw_AFib: settings.sw_AFib.boolValue,
            sw_AMPM: settings.sw_AMPM.boolValue,
            sw_Kpa: settings.sw_Kpa.boolValue,
            restTime: settings.restTime.intValue,
            intervalTime: settings.intervalTime.intValue,
            autoMeasureNumber: settings.autoMeasureNumber.intValue)
        log.T(text: "Writing setting values:\(settings.parseDictionary())")
    }

    func readDeviceIDAndInfo() {
        watchBPM1.readDeviceIDAndInfo()
        log.T(text: "Reading device ID and info")
    }

    func readDeviceTime() {
        watchBPM1.readDeviceTime()
        log.T(text: "Reading device time")
    }

    func writeDeviceTime() {
        watchBPM1.writeDeviceTime()
        log.T(text: "Writing device time")
    }

    func readFunctionSettingValue() {
        watchBPM1.readFunctionSettingValue()
        log.T(text: "Reading function setting value")
    }

    func readBTModuleName() {
        watchBPM1.readBTModuleName()
        log.T(text: "Reading Bluetooth module name")
    }

}
