//
//  WatchBPO3ViewModel.swift
//  MicroLifeDeviceSDK-Demo_Swift
//
//  Created by 吳憲有 on 2024/5/20.
//

import Foundation

@Observable
class WatchBPO3ViewModel {
        
    var deviceConnected = watchBPO3.deviceConnected
        
    var CBPindex = 0
    
    var historyMeasuremeCount = 0

    var dformat = Dformat.fullCBPRaw

    var showSettings = false
    
    var settings = MicroLifeWatchBPSettingValues()
    
    func setupBLESDK() {
        
        log.clear()

        sdk.cancelAllConnect()
        
        watchBPO3.autoConnect(true)
        
        watchBPO3.getDidUpdateStateBlock { state in
            log.T(text: "isBLEStatePoweredOn: \(state)")
        } scanDeviceBlock: { checkMacType, device in
            if let microLifeDevice = device as? MicroLifeDevice {
                log.T(text: "ScanDevice:\nName:\(microLifeDevice.name) MacType:\(checkMacType.description) RSSI:\(microLifeDevice.rssi)\nUUID:\(microLifeDevice.uuid)")
            }
        } cancelScanBlock: {
            log.T(text: "Cancel Scan")
        } connectDeviceStateBlock: { device, _, _, state, error in
            log.T(text: "Device: \(device)\nPeripheral State: \(state)")
            log.E(error: error)
        } cancelAllDevicesConnectionBlock: {
            log.T(text: "Cancel All Devices Connection")
            self.settings = MicroLifeWatchBPSettingValues()
            self.historyMeasuremeCount = 0
            self.showSettings = false
            watchBPO3.startScan()
        }
        
        watchBPO3.deviceConnectedBlock {
            self.deviceConnected = watchBPO3.deviceConnected
        }
        
        watchBPO3.getReadDataModelBlock { CMD, model, error in
            log.M(CMD: CMD, model: model)
            log.E(error: error)
            guard let command = CMDWatchBPO3(rawValue: CMD) else {
                return
            }
            switch command {
            case .bpo3ReadHistorys:
            // Handle reading history data
                if let data = model as? MicroLifeWatchBPDRecord {
                    self.historyMeasuremeCount = data.historyMeasuremeNumber.intValue
                    print("historyMeasuremeCount: \(self.historyMeasuremeCount)")
                }
                break
            case .bpo3ReadCBPData:
            // Handle reading CBP data
                break
            case .bpo3ClearAllHistorys:
            // Handle clearing all history data
                break
            case .bpo3Disconnect:
            // Handle disconnecting
                break
            case .bpo3ReadUserAndVersionData:
            // Handle reading user and version data
                break
            case .bpo3WriteUser:
            // Handle writing user data
                break
            case .bpo3ReadDeviceInfo:
            // Handle reading device information
                break
            case .bpo3ReadDeviceTime:
            // Handle reading device time
                break
            case .bpo3WriteDeviceTime:
            // Handle writing device time
                break
            case .bpo3ReadSerialNumber:
            // Handle reading serial number
                break
            case .bpo3ReadFunctionSettingValue:
            // Handle reading function setting value
                break
            case .bpo3ReadBTModuleName:
            // Handle reading BT module name
                break
            case .bpo35SReadSettingValues:
            // Handle reading setting values 5S
                self.showSettings = true
                self.settings = model as! MicroLifeWatchBPSettingValues
                break
            case .bpo3ReadSettingValues:
            // Handle reading setting values
                self.showSettings = true
                self.settings = model as! MicroLifeWatchBPSettingValues
                break
            case .bpo3WriteSettingValues:
            // Handle writing setting values
                break
            case .bpo35SWriteSettingValues:
            // Handle writing setting values 5S
                break
            default:
                break
            }
        }
        
        watchBPO3.getValidationErrorBlock { item, info, value in
            log.T(text: "\(item)\ninfo:\(info)\nvalue:\(value)")
        }
                
    }
    
    func refreshData() {
        log.clear()
        log.T(text: "refreshData")
        disconnectDevice()
        startScan()
    }
    
    func startScan() {
        watchBPO3.startScan()
        log.T(text: "Start Scan")
    }

    func cancelScan() {
        watchBPO3.cancelScan()
        log.T(text: "Cancel Scan")
    }
    
    func disconnectDevice() {
        if deviceConnected {
            watchBPO3.disconnect()
            watchBPO3.disconnectDevice()
        }
        log.T(text: deviceConnected ? "unConnect" : "disconnectDevice")
    }
    
    /// WatchBPO3 command
    func readAllHistorys() {
        watchBPO3.readAllHistorys()
        log.T(text: "Reading all history data")
    }

    func readCBPData() {
        watchBPO3.readCBPData(with: CBPindex, dformat: MicroLifeDeviceSDK.Dformat(rawValue: dformat.rawValue) ?? MicroLifeDeviceSDK.Dformat.MicroLifeDFullCBPRaw)
        log.T(text: "Reading CBP data \(CBPindex) dformat \(dformat.rawValue)")
    }

    func clearAllHistorys() {
        watchBPO3.clearAllHistorys()
        log.T(text: "Clearing all history data")
    }

    func disconnect() {
        watchBPO3.disconnect()
        log.T(text: "Disconnect")
    }

    func readUserAndVersionData() {
        watchBPO3.readUserAndVersionData()
        log.T(text: "Reading user and version data")
    }

    func writeUserID() {
        watchBPO3.writeUserID(appInfo.user.bpmUserID)
        log.T(text: "Writing user ID")
    }

    func readSettingValues() {
        watchBPO3.readSettingValues()
        log.T(text: "Reading setting values")
    }

    func writeSettingValues() {
        let checkSettingValues = settings.checkSettingValues()
        if checkSettingValues {
            if settings.settingValues.count > 0 {
                watchBPO3.writeSettingValues(
                    withSettingValues: settings.settingValues,
                    hi_infPressure: settings.hi_infPressure,
                    sw_checkhide: settings.sw_checkhide.boolValue)
            } else {
                watchBPO3.writeSettingValues(
                    withSelectedABPMStart: settings.abpmStart.intValue,
                    abpmEnd: settings.abpmEnd.intValue,
                    abpmInt_first: settings.abpmInt_first,
                    abpmInt_second: settings.abpmInt_second,
                    hi_infPressure: settings.hi_infPressure,
                    sw_checkhide: settings.sw_checkhide.boolValue,
                    sw_SEL_silent: settings.sw_SEL_silent.boolValue,
                    cbp_zone1_meas_off: settings.cbp_zone1_meas_off.boolValue,
                    cbp_zone2_meas_off: settings.cbp_zone2_meas_off.boolValue,
                    cbpInt_first: settings.cbpInt_first,
                    cbpInt_second: settings.cbpInt_second)
            }
        }
        log.T(text: "writeSettingValues:\(checkSettingValues ? settings.parseDictionary().description : "NO")")
    }

    func readDeviceIDAndInfo() {
        watchBPO3.readDeviceIDAndInfo()
        log.T(text: "Reading device ID and information")
    }

    func readDeviceTime() {
        watchBPO3.readDeviceTime()
        log.T(text: "Reading device time")
    }

    func writeDeviceTime() {
        watchBPO3.writeDeviceTime()
        log.T(text: "Writing device time")
    }

    func readSerialNumber() {
        watchBPO3.readSerialNumber()
        log.T(text: "Reading serial number")
    }

    func readFunctionSettingValue() {
        watchBPO3.readFunctionSettingValue()
        log.T(text: "Reading function setting value")
    }

    func readBTModuleName() {
        watchBPO3.readBTModuleName()
        log.T(text: "Reading Bluetooth module name")
    }

}
