//
//  WatchBPOfficeViewModel.swift
//  MicroLifeDeviceSDK-Demo_Swift
//
//  Created by 吳憲有 on 2024/5/21.
//

import Foundation

@Observable
class WatchBPOfficeViewModel {
    var deviceConnected = watchBPOffice.deviceConnected

    var CBPindex = 0
    
    var historyMeasuremeCount = 0

    var dformat = Dformat.fullCBPRaw

    var showSettings = false

    var settings = MicroLifeWatchBPSettingValues()

    func setupBLESDK() {
        log.clear()

        sdk.cancelAllConnect()

        watchBPOffice.autoConnect(true)

        watchBPOffice.getDidUpdateStateBlock { state in
            log.T(text: "isBLEStatePoweredOn: \(state)")
        } scanDeviceBlock: { checkMacType, device in
            if let microLifeDevice = device as? MicroLifeDevice {
                log.T(text: "ScanDevice:\nName:\(microLifeDevice.name) MacType:\(checkMacType.description) RSSI:\(microLifeDevice.rssi)\nUUID:\(microLifeDevice.uuid)")
            }
        } cancelScanBlock: {
            log.T(text: "Cancel Scan")
        } connectDeviceStateBlock: { device, _, _, state, error in
            log.T(text: "Device: \(device)\nPeripheral State: \(state)")
            log.E(error: error)
        } cancelAllDevicesConnectionBlock: {
            log.T(text: "Cancel All Devices Connection")
            self.settings = MicroLifeWatchBPSettingValues()
            self.historyMeasuremeCount = 0
            watchBPOffice.startScan()
        }

        watchBPOffice.deviceConnectedBlock {
            self.deviceConnected = watchBPOffice.deviceConnected
        }

        watchBPOffice.getReadDataModelBlock { CMD, model, error in
            log.M(CMD: CMD, model: model)
            log.E(error: error)
            guard let command = CMDWatchBPOffice(rawValue: CMD) else {
                return
            }
            switch command {
            case .readHistorys:
                // Handle reading history data...
                if let data = model as? MicroLifeWatchBPDRecord {
                    self.historyMeasuremeCount = data.historyMeasuremeNumber.intValue
                }
            case .readCBPData:
                // Handle reading CBP data...
                break
            case .clearAllHistorys:
                // Handle clearing all history data...
                break
            case .disconnect:
                // Handle disconnecting...
                break
            case .readUserAndVersionData:
                // Handle reading user and version data...
                break
            case .writeUser:
                // Handle writing user data...
                break
            case .readSettingValues:
                // Handle reading setting values...
                self.settings = model as! MicroLifeWatchBPSettingValues
            case .writeSettingValues:
                // Handle writing setting values...
                break
            case .readDeviceInfo:
                // Handle reading device info...
                break
            case .readDeviceTime:
                // Handle reading device time...
                break
            case .writeDeviceTime:
                // Handle writing device time...
                break
            case .readFunctionSettingValue:
                // Handle reading function setting value...
                break
            case .readBTModuleName:
                // Handle reading Bluetooth module name...
                break
            case .startRemoteMeasurement:
                // Handle starting remote measurement...
                break
            case .stopRemoteMeasurement:
                // Handle stopping remote measurement...
                break
            case .sendRemoteMeasurementStatusEvery5seconds:
                // Handle sending remote measurement status every 5 seconds...
                break
            case .sendMeasurementResultsForEachMeasurement:
                // Handle sending measurement results for each measurement...
                break
            default:
                break
            }
        }

        watchBPOffice.getValidationErrorBlock { item, info, value in
            log.T(text: "\(item)\ninfo:\(info)\nvalue:\(value)")
        }
    }

    func refreshData() {
        log.clear()
        log.T(text: "refreshData")
        disconnectDevice()
        startScan()
    }

    func startScan() {
        watchBPOffice.startScan()
        log.T(text: "Start Scan")
    }

    func cancelScan() {
        watchBPOffice.cancelScan()
        log.T(text: "Cancel Scan")
    }

    func disconnectDevice() {
        if deviceConnected {
            watchBPOffice.disconnect()
            watchBPOffice.disconnectDevice()
        }
        log.T(text: deviceConnected ? "unConnect" : "disconnectDevice")
    }

    /// WatchBPOffice command
    func readAllHistorys() {
        watchBPOffice.readAllHistorys()
        log.T(text: "Reading all history data")
    }

    func readCBPData() {
        watchBPOffice.readCBPData(with: CBPindex, dformat: MicroLifeDeviceSDK.Dformat(rawValue: dformat.rawValue) ?? MicroLifeDeviceSDK.Dformat.MicroLifeDFullCBPRaw)
        log.T(text: "Reading CBP data \(CBPindex) dformat \(dformat.rawValue)")
    }

    func clearAllHistorys() {
        watchBPOffice.clearAllHistorys()
        log.T(text: "Clearing all history data")
    }

    func disconnect() {
        watchBPOffice.disconnect()
        log.T(text: "Disconnecting from device")
    }

    func readUserAndVersionData() {
        watchBPOffice.readUserAndVersionData()
        log.T(text: "Reading user and version data")
    }

    func writeUserID() {
        watchBPOffice.writeUserID(appInfo.user.bpmUserID)
        log.T(text: "Writing user ID: \(appInfo.user.bpmUserID)")
    }

    func readSettingValues() {
        watchBPOffice.readSettingValues()
        log.T(text: "Reading setting values")
    }

    func writeSettingValues() {
        watchBPOffice.writeSettingValues(
            withSelectedAUS_HI_infPressure: settings.aus_HI_infPressure,
            hi_infPressure: settings.hi_infPressure,
            sw_AUTO_hide: settings.sw_AUS_Hide.boolValue,
            sw_SEL_silent: settings.sw_SEL_silent.boolValue,
            sw_AUS_Hide: settings.sw_AUS_Hide.boolValue,
            sw_AVG_no_include_first: settings.sw_AVG_no_include_first.boolValue,
            sw_CBP: settings.sw_CBP.boolValue,
            sw_AFib: settings.sw_AFib.boolValue,
            sw_AMPM: settings.sw_AMPM.boolValue,
            sw_Kpa: settings.sw_Kpa.boolValue,
            restTime: settings.restTime.intValue,
            intervalTime: settings.intervalTime.intValue,
            autoMeasureNumber: settings.autoMeasureNumber.intValue)
        log.T(text: "Writing setting values:\(settings.parseDictionary())")
    }

    func readDeviceIDAndInfo() {
        watchBPOffice.readDeviceIDAndInfo()
        log.T(text: "Reading device ID and info")
    }

    func readDeviceTime() {
        watchBPOffice.readDeviceTime()
        log.T(text: "Reading device time")
    }

    func writeDeviceTime() {
        watchBPOffice.writeDeviceTime()
        log.T(text: "Writing device time")
    }

    func readFunctionSettingValue() {
        watchBPOffice.readFunctionSettingValue()
        log.T(text: "Reading function setting value")
    }

    func readBTModuleName() {
        watchBPOffice.readBTModuleName()
        log.T(text: "Reading Bluetooth module name")
    }

    func startRemoteMeasurement() {
        watchBPOffice.startRemoteMeasurementWhtihCBPFunction(MicroLifeDeviceSDK.Dformat(rawValue: dformat.rawValue) ?? MicroLifeDeviceSDK.Dformat.MicroLifeDFullCBPRaw)
        log.T(text: "Starting remote measurement with \(dformat.rawValue)")
    }

    func stopRemoteMeasurement() {
        watchBPOffice.stopRemoteMeasurement()
        log.T(text: "Stopping remote measurement")
    }
}
