//
//  WeightView.swift
//  MicroLifeDeviceSDK-Demo_Swift
//
//  Created by 吳憲有 on 2024/5/10.
//

import SwiftUI

struct WeightView: View {
    
    @Environment(AppInfo.self) var appInfo
    
    @Environment(\.horizontalSizeClass) var horizontalSizeClass
    
    @Environment(\.verticalSizeClass) var verticalSizeClass
    
    var viewModel = WeightViewModel()
    
    var menuInfo: MenuInfo
    
    var body: some View {
        let orientation = DeviceOrientation.checkOrientation(horizontalSizeClass: horizontalSizeClass, verticalSizeClass: verticalSizeClass)
        ZStack {
            switch orientation {
            case .iPhoneHorizontal, .OtherDevice:
                HorizontalLayoutView(scale: 0.75) {
                    viewModel.startScan()
                } cancelScanAction: {
                    viewModel.cancelScan()
                } disconnectDeviceAction: {
                    viewModel.disconnectDevice()
                } commandViewContent: {
                    CustomWeightView(viewModel: viewModel)
                }
            case .iPhoneVertical, .iPad:
                VerticalLayoutView(scale: 0.75) {
                    viewModel.startScan()
                } cancelScanAction: {
                    viewModel.cancelScan()
                } disconnectDeviceAction: {
                    viewModel.disconnectDevice()
                } commandViewContent: {
                    CustomWeightView(viewModel: viewModel)
                }
            }
        }
        .toolbar {
            CustomToolbarItem(
                filePath: log.filePath,
                fileName: log.fileName,
                refreshAction: {
                    viewModel.refreshData()
                })
        }
        .navigationTitle(menuInfo.title)
        .padding(.horizontal, 8)
        .onChange(of: appInfo.showViewType) {
            if appInfo.showViewType == .weighingScale {
                viewModel.setupBLESDK()
            }
        }
        .onAppear {
            appInfo.showViewType = .weighingScale
            viewModel.setupBLESDK()
        }
        .onDisappear {
            appInfo.showViewType = .menu
            viewModel.disconnectDevice()
        }
    }
}

struct CustomWeightView: View {
    
    var viewModel: WeightViewModel

    var body: some View {
        VStack(alignment: .trailing, spacing: 16) {
            Button(action: {
                writeUserData()
            }) {
                Text("Write User Data")
            }
            
            Button(action: {
                viewModel.clearAllUsers()
            }) {
                Text("Clear All Users")
            }
            
            Button(action: {
                viewModel.readHistorys()
            }) {
                Text("Read History")
            }
            
            Button(action: {
                viewModel.disconnect()
            }) {
                Text("Disconnect")
            }
        }
        .padding()
    }
}


#Preview {
    WeightView(menuInfo: MenuInfo())
}
