//
//  MenuViewController.m
//  MicroLifeDeviceSDK-Demo
//
//  Created by 吳憲有 on 2022/12/8.
//  Copyright © 2022 WiFi@MBP. All rights reserved.
//

#import "MenuViewController.h"

@interface MenuViewController ()

@end

@implementation MenuViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.menuTableView.backgroundColor = [UIColor whiteColor];
    self.menuTableView.refreshControl = self.menuRefresh;
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [self BLESDK];
    [self loadData];
}

- (void)BLESDK {
    kWeakSelf(self)
    [self.sdk cancelAllConnect];
    self.bp3g = [MicroLife3GBP shareWhithAuthorization:SDK_code Key:SDK_key Secret:SDK_secret];
    [self.bp3g autoConnect:NO];
    self.bp4g = [MicroLife4GBP shareWhithAuthorization:SDK_code Key:SDK_key Secret:SDK_secret];
    [self.bp4g autoConnect:NO];
    self.temperature = [MicroLifeTemperature shareWhithAuthorization:SDK_code Key:SDK_key Secret:SDK_secret];
    [self.temperature addDeviceModel:@[@"FR1MZ1-B",@"NC Thermometer2"]];
    [self.temperature autoConnect:NO];
    self.oxygen = [MicroLifeOxygen shareWhithAuthorization:SDK_code Key:SDK_key Secret:SDK_secret];
    [self.oxygen autoConnect:NO];
    self.weight = [MicroLifeWeight shareWhithAuthorization:SDK_code Key:SDK_key Secret:SDK_secret];
    [self.weight autoConnect:NO];
    self.bgm800 = [MicroLifeBGM800 shareWhithAuthorization:SDK_code Key:SDK_key Secret:SDK_secret];
    [self.bgm800 autoConnect:NO];
    self.bgm700 = [MicroLifeBGM700 shareWhithAuthorization:SDK_code Key:SDK_key Secret:SDK_secret];
    [self.bgm700 autoConnect:NO];
    self.watchBPOffice = [MicroLifeWatchBPOffice shareWhithAuthorization:SDK_code Key:SDK_key Secret:SDK_secret];
    [self.watchBPOffice autoConnect:NO];
    self.watchBPM1 = [MicroLifeWatchBPM1 shareWhithAuthorization:SDK_code Key:SDK_key Secret:SDK_secret];
    [self.watchBPM1 autoConnect:NO];
    self.watchBPHome = [MicroLifeWatchBPHome shareWhithAuthorization:SDK_code Key:SDK_key Secret:SDK_secret];
    [self.watchBPHome autoConnect:NO];
    self.watchBPO3 = [MicroLifeWatchBPO3 shareWhithAuthorization:SDK_code Key:SDK_key Secret:SDK_secret];
    [self.watchBPO3 autoConnect:NO];
    self.peakFlowMeter = [MicroLifePFM shareWhithAuthorization:SDK_code Key:SDK_key Secret:SDK_secret];
    [self.peakFlowMeter autoConnect:NO];
    self.electrocardiography = [MicroLifeECG shareWhithAuthorization:SDK_code Key:SDK_key Secret:SDK_secret];
    [self.electrocardiography addDeviceModel:@[@"EBPHKW2"]];
    [self.electrocardiography autoConnect:NO];
    [self.sdk device:@[self.bp3g,self.bp4g,self.temperature,self.oxygen,self.weight,self.bgm800,self.bgm700,self.watchBPOffice,self.watchBPM1,self.watchBPHome,self.watchBPO3,self.peakFlowMeter,self.electrocardiography]];
    [self.sdk getDidUpdateStateBlock:^(CBManagerState state) {
        
    }];
    [self.sdk getScanDeviceBlock:^(NSInteger checkMacType, id  _Nonnull device) {
        if ([device isKindOfClass:[MicroLifeDevice class]]) {
            [weakself addDevice:device];
        }
    } CancelScanBlock:^() {
        
    }];
}

- (void)addDevice:(MicroLifeDevice *)device {
    for (MenuInfo *m in self.menus) {
        if (m.deviceType == device.deviceType) {
            [m addDevice:device];
        }
    }
    kWeakSelf(self)
    dispatch_async(dispatch_get_main_queue(), ^{
        [weakself.menuTableView reloadData];
    });
}

#pragma mark - RefreshControl
- (UIRefreshControl *)menuRefresh {
    if (!_menuRefresh) {
        self.menuRefresh = [UIRefreshControl new];
        [self.menuRefresh addTarget:self action:@selector(loadData) forControlEvents:UIControlEventValueChanged];
    }
    return _menuRefresh;
}

- (void)loadData {
    for (MenuInfo *m in self.menus) {
        [m removalInfo];
    }
    kWeakSelf(self)
    dispatch_async(dispatch_get_main_queue(), ^{
        [weakself.menuTableView reloadData];
    });
    [self.sdk cancelScan];
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        // stop refreshing after 1 seconds
        [weakself.sdk startScan];
        [weakself.menuRefresh endRefreshing];
     });
}

#pragma mark -
- (NSMutableArray *)menus {
    if (!_menus) {
        self.menus = [MenuInfo menus];
    }
    return _menus;
}

#pragma mark - UITableViewDelegate,UITableViewDataSource
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.menus.count;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 100;
}
    
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"Cell"];
    if (!cell) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"Cell"];
    }
    MenuInfo *info = [self.menus objectAtIndex:indexPath.row];
    if (info.name && info.UUID && info.RSSI) {
        cell.textLabel.text = [NSString stringWithFormat:@"%@\nName:%@ RSSI:%@\nUUID:%@ ",info.title,info.name,info.RSSI,info.UUID];
    } else {
        cell.textLabel.text = info.title;
    }
    cell.textLabel.numberOfLines = 3;
    cell.textLabel.textAlignment = NSTextAlignmentCenter;
    cell.textLabel.adjustsFontSizeToFitWidth = YES;
    cell.backgroundColor = info.backgroundColor;
    cell.textLabel.textColor = info.textColor;
    return cell;
}
    
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath{
    MenuInfo *info = [self.menus objectAtIndex:indexPath.row];
    switch (info.deviceType) {
        case 100:
            [self presentVC:@"OneViewController" Info:info];
            break;
        case 101:
            [self presentVC:@"CloudViewController" Info:info];
            break;
        case 102:
            if ([MFMailComposeViewController canSendMail]) {
                [self emailLog];
            } else {
                [self shareLog];
            }
            break;
        case DeviceType3GBP:
        case DeviceType4GBP:
            [self presentVC:@"BPMViewController" Info:info];
            break;
        case DeviceTypeWeight:
            [self presentVC:@"WEIViewController" Info:info];
            break;
        case DeviceTypeTemperature:
            [self presentVC:@"TEMPViewController" Info:info];
            break;
        case DeviceTypeOxygen:
            [self presentVC:@"OxygenViewController" Info:info];
            break;
        case DeviceTypeECG:
            [self presentVC:@"ECGViewController" Info:info];
            break;
        case DeviceTypeBGM700:
            [self presentVC:@"BSViewController" Info:info];
            break;
        case DeviceTypeWatchBPHome:
            [self presentVC:@"WBPViewController" Info:info];
            break;
        case DeviceTypeWatchBPO3:
            [self presentVC:@"WBO3ViewController" Info:info];
            break;
        case DeviceTypeWatchBPM1:
            [self presentVC:@"M1ViewController" Info:info];
            break;
        case DeviceTypeWatchBPOffice:
            [self presentVC:@"WBOViewController" Info:info];
            break;
        case DeviceTypePFM:
            [self presentVC:@"PFMViewController" Info:info];
            break;
        case DeviceTypePPG:
            [self presentVC:@"PPGViewController" Info:info];
            break;
        default:
            break;
    }
}

- (void)presentVC:(NSString *)identifier Info:(MenuInfo *)info {
    UIStoryboard *storyboard = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
    RootViewController *vc = [storyboard instantiateViewControllerWithIdentifier:identifier];
    vc.menuInfo = info;
    [self.navigationController pushViewController:vc animated:YES];
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end

