//
//  ECGParamter.m
//  ECGChartView
//
//  Created by Mac on 2021/6/4.
//

#import "YOECGParamter.h"
#import <UIKit/UIKit.h>
@implementation YOECGParamter

-(instancetype)init{
    if (self = [super init]) {
        self.speed = 25;
        self.voltageSpecifications = 10;
        self.sampleFrequency = 256;
        self.oneGridSize = [self oneMillimetrePt] * 5;
        self.voltageUnit = YOECGChartViewVoltageUnitMilliVolt;
    }
    return self;
}

-(float)secodePerWidth{
    return self.speed;
}

-(float)onePointWidth{
    /**
     1 格子 = self.oneGridSize / 5.0
     1 格子 = 1mm
     1s走纸距离(格子数) = 速度 * 时间 => self.speed * 1 =  self.speed * 1
     1s走纸点数 = self.sampleFrequency;
     每个点的 pt 为 格子数 * 一个格子的距离 => self.speed * self.oneGridSize / 5.0
     */
    return self.speed * self.oneGridSize / 5.0 / self.sampleFrequency ;
}

-(float)onePointHeight{
    /**
     
     voltageSpecifications 纸张规格 10 mm/mV (一小格 0.1mV)  【不建议修改】
     
     1 格子 = self.oneGridSize / 5.0
     1 格子 = 1mm / 0.1mV
     1 格子 = 1mm / voltageSpecifications mm/mV  = (1.0 / voltageSpecifications )mV
     
     每个点的 pt 为 格子数 * 一个格子的距离 => self.speed * self.oneGridSize / 5.0
     */
    return self.oneGridSize / 5.0 / (1.0 / self.voltageSpecifications); // 每个距离的mV  =>  pt / mV
    
}


//-(float)oneMillimetrePt{
//    CGFloat sc_w = [[UIScreen mainScreen] bounds].size.width;
//    CGFloat sc_h = [[UIScreen mainScreen] bounds].size.height;
//    CGFloat sc_s;
//    CGFloat ff = [[UIScreen mainScreen] nativeBounds].size.height;
//    
//    if (ff == 1136) {
//        sc_s = 4.0;
//    }else if(ff == 1334.0){
//        sc_s = 4.7;
//    }else if (ff == 2340){
//        sc_s = 5.4;
//    }else if (ff== 1920){
//        sc_s = 5.5;
//    }else if (ff== 2436){
//        sc_s = 5.8;
//    }else if (ff == 1792){
//        sc_s = 6.1;
//    }else if (ff == 2532){
//        sc_s = 6.1;
//    }else if (ff == 2688){
//        sc_s = 6.5;
//    }else if (ff == 2778){
//        sc_s = 6.7;
//    }else{
//        sc_s = 5.8;
//    }
//    
//    //1mm米的像素点
//    float ppm = sqrt(sc_w * sc_w + sc_h * sc_h)/(sc_s * 25.4) ;//mm
//    return ppm;// / [UIScreen mainScreen].scale;
//}

-(float)oneMillimetrePt {
    // 获取屏幕逻辑尺寸和原生尺寸
    CGFloat sc_w = [[UIScreen mainScreen] bounds].size.width;
    CGFloat sc_h = [[UIScreen mainScreen] bounds].size.height;
    CGFloat ff = [[UIScreen mainScreen] nativeBounds].size.height;

    // 映射屏幕高度到屏幕尺寸，并附加对应型号的说明
    NSDictionary<NSNumber *, NSNumber *> *screenSizeMap = @{
        @1136 : @4.0,   // iPhone 5, 5c, 5s, SE (1st generation)
        @1334 : @4.7,   // iPhone 6, 6s, 7, 8
        @1920 : @5.5,   // iPhone 6 Plus, 6s Plus, 7 Plus, 8 Plus
        @2340 : @5.4,   // iPhone 12 mini, 13 mini
        @2436 : @5.8,   // iPhone X, XS, 11 Pro
        @1792 : @6.1,   // iPhone XR, 11
        @2532 : @6.1,   // iPhone 12, 12 Pro, 13, 13 Pro, 14, 15, 15 Plus
        @2688 : @6.5,   // iPhone XS Max, 11 Pro Max
        @2778 : @6.7,   // iPhone 12 Pro Max, 13 Pro Max, 14 Plus, 15 Plus
        @2556 : @6.1,   // iPhone 14 Pro, 15 Pro
        @2796 : @6.7,   // iPhone 14 Pro Max, 15 Pro Max
        @2388 : @11.0,  // iPad Pro 11" (1st, 2nd, 3rd, 4th generation)
        @2732 : @12.9,  // iPad Pro 12.9" (1st, 2nd, 3rd, 4th, 5th generation)
        @2160 : @10.2,  // iPad (7th, 8th, 9th generation)
        @2360 : @10.9,  // iPad Air (4th, 5th generation)
        @2224 : @10.5,  // iPad Pro 10.5", iPad Air (3rd generation)
        @2048 : @9.7    // iPad (1st to 6th generation), iPad Air (1st, 2nd generation), iPad Pro 9.7"
    };

    // 获取屏幕尺寸，如果字典中没有找到匹配的分辨率，使用默认值5.8英寸
    NSNumber *screenSizeNum = screenSizeMap[@(ff)];
    CGFloat sc_s = screenSizeNum ? screenSizeNum.floatValue : 5.8;

    // 计算每毫米的像素点
    float ppm = sqrt(sc_w * sc_w + sc_h * sc_h) / (sc_s * 25.4); // mm
    return ppm;
}




-(BOOL)parameterIsRight{
    return self.speed * self.sampleFrequency * self.voltageSpecifications * self.oneGridSize > 0;
}


@end
