//
//  MicroLifeDataModel.h
//  MicroLifeDeviceSDK
// 1. Analysis tool
// 2. Analysis of each device instruction (rewritten by each device)
//  Created by willy.wu on 2020/11/18.
//

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>
#import <objc/runtime.h>

typedef NS_ENUM(NSInteger , DeviceType) {
    DeviceTypeUnknown,
    DeviceType3GBP,
    DeviceType4GBP,
    DeviceTypeWeight,
    DeviceTypeTemperature,
    DeviceTypeOxygen,
    DeviceTypeWatchBPHome,
    DeviceTypeWatchBPO3,
    DeviceTypeWatchBPOffice,
    DeviceTypeECG,
    DeviceTypeWatchBPM1,
    DeviceTypeBGM700,
    DeviceTypeBGM800,
    DeviceTypePFM,
    DeviceTypePPG,
};

/**
 STATUS:

 - MicroLifeMeasurementWait: device wait countdown for next measurement
 - MicroLifeMeasurementStart: device is start BP measurement
 - MicroLifeMeasurementStop: manual press I/O to stop measurement
 */
typedef NS_ENUM(NSInteger, STATUS) {
    MicroLifeMeasurementWait    = 0x01,
    MicroLifeMeasurementStart   = 0x02,
    MicroLifeMeasurementStop    = 0x04,
};

/**
Dformat:Data format which APP requested.

- MicroLifeDNoCBPRaw: No CBP raw data
- MicroLifeDLowCBPRaw: low resolution CBP data (sampling rate =16Hz)
- MicroLifeDFullCBPRaw: full CBP raw data (sampling rate=256Hz)
*/
typedef NS_ENUM(NSInteger, Dformat) {
    MicroLifeDNoCBPRaw      = 0x00,
    MicroLifeDLowCBPRaw     = 0x01,
    MicroLifeDFullCBPRaw    = 0x03
};

/**
 Highest inflation pressure of Auto mode: Auto mode, 140, 160, 180, 200, 220, 240, 280

 The device inflates the cuff using fuzzy logic to proper cuff pressure; the Highest Inflation Pressure is considered as a safeguard pressure

 - MicroLifeHIinfPressureAuto: Auto mode
 - MicroLifeHIinfPressure140: 140
 - MicroLifeHIinfPressure160: 160
 - MicroLifeHIinfPressure180: 180
 - MicroLifeHIinfPressure200: 200
 - MicroLifeHIinfPressure220: 220
 - MicroLifeHIinfPressure240: 240
 - MicroLifeHIinfPressure280: 280
*/
typedef NS_ENUM(NSInteger, HIinfPressure) {
    MicroLifeHIinfPressureAuto  = 0,
    MicroLifeHIinfPressure140   = 140,
    MicroLifeHIinfPressure160   = 160,
    MicroLifeHIinfPressure180   = 180,
    MicroLifeHIinfPressure200   = 200,
    MicroLifeHIinfPressure220   = 220,
    MicroLifeHIinfPressure240   = 240,
    MicroLifeHIinfPressure280   = 280
};

typedef NS_ENUM(NSInteger, Interval) {
    MicroLifeInterval5  = 5,
    MicroLifeInterval10  = 10,
    MicroLifeInterval15  = 15,
    MicroLifeInterval20  = 20,
    MicroLifeInterval30  = 30,
    MicroLifeInterval60  = 60,
    MicroLifeIntervalOff  = 255
};

typedef NS_ENUM(NSInteger, MeasurementTime) {
    MicroLifeMeasurementTimeOff  = 0,
    MicroLifeMeasurementTimel5  = 5,
    MicroLifeMeasurementTime10  = 10,
    MicroLifeMeasurementTime15  = 15,
    MicroLifeMeasurementTime20  = 20,
    MicroLifeMeasurementTime30  = 30,
    MicroLifeMeasurementTime60  = 60
};

typedef NS_ENUM(NSInteger , TransmitRate) {
    TransmitRate256Hz = 1,
    TransmitRate512Hz = 2
};

/**
 User gender

 - MicroLifeUserGenderMale: Male
 - MicroLifeUserGenderFemale: Female
 - MicroLifeUserGenderHideyoshi: Diverse
 */
typedef NS_ENUM(NSInteger, MicroLifeUserGender) {
    MicroLifeUserGenderMale,
    MicroLifeUserGenderFemale,
    MicroLifeUserGenderHideyoshi
};

/**
 User's exercise type

 - MicroLifeUserRoleTypeNormal: Normal
 - MicroLifeUserRoleTypeSportsman: Athlete
 */
typedef NS_ENUM(NSInteger, MicroLifeUserRoleType) {
    MicroLifeUserRoleTypeNormal = 0x01,
    MicroLifeUserRoleTypeSportsman = 0x02
};

/**
 Weight unit (kg/lb)

 - MicroLifeEBUnitTypeKG: kg
 - MicroLifeEBUnitTypeLB: lb
 */
typedef NS_ENUM(NSInteger, MicroLifeEBUnitType) {
    MicroLifeEBUnitTypeKG,
    MicroLifeEBUnitTypeLB,
};


typedef NS_ENUM(NSInteger, MAMInterval) {
    MicroLifeMAMInterval15  = 15,
    MicroLifeMAMInterval30  = 30,
    MicroLifeMAMInterval60  = 60,
    MicroLifeMAMInterval120  = 120
};

typedef NS_ENUM(NSInteger, SsampleTime) {
    MicroLifeSsampleTimeNA = 0,
    MicroLifeSsampleTime30 = 1,
    MicroLifeSsampleTime60 = 2,
    MicroLifeSsampleTime540 = 9,
    MicroLifeSsampleTime1830 = 31
};

typedef NS_ENUM(NSInteger, SampleRate) {
    MicroLifeSampleRate128 = 0,
    MicroLifeSampleRate256 = 1,
    MicroLifeSampleRate512 = 2
};

/// ECG Status:
/// Note: Refer to 2.1.3 ECG measurement flowchart, device will auto update device status (0x05~0x08) during ECG measurement. But if stop ECG measurement, device will stop auto to update status.
/// Device status    Raw data    Description
/// manual enter Bluetooth    0x01    User press i/o to enter Bluetooth mode.
/// Detect new blood pressure data    0x02    After blood pressure measurement (not trigger ECG) , auto enter Bluetooth mode.
/// Detect new blood pressure data, wait ECG trigger    0x03    After blood pressure measurement with arrhythmia, auto enter Bluetooth mode and wait ECG command (25h).
/// Only ECG mode, , wait ECG trigger    0x04    Starts only ECG measurement, wait ECG start command (25h).
/// Wait lead-on stable    0x05    Receive ECG start command 25h, wait user lead-on. Update this command (20h) every 500ms until timeout (30 seconds).
/// Start ECG measurement    0x06    Detect lead on stable, ready to start ECG measurement. Than auto transmit ECG raw data(package type=69h or 70h)
/// Stop ECG measurement    0x07    Stop ECG measurement caused by lead-off, remote stop, user stop or transmit overflow.
/// Succeed and store memory    0x08    Completed measurement and store memory
/// Factory mode    0x09
typedef NS_ENUM(NSInteger, ECGStatus) {
    MicroLifeECGStatusManualEnterBluetooth = 0x01,
    MicroLifeECGStatusDetectNewBloodPressureData = 0x02,
    MicroLifeECGStatusDetectNewBloodPressureDataWaitECGTrigger = 0x03,
    MicroLifeECGStatusOnlyECGModeWaitECGTrigger = 0x04,
    MicroLifeECGStatusWaitLeadOnStable = 0x05,
    MicroLifeECGStatusStartECGMeasurement = 0x06,
    MicroLifeECGStatusStopECGMeasurement = 0x07,
    MicroLifeECGStatusSucceedAndStoreMemory = 0x08,
    MicroLifeECGStatusFactoryMode = 0x09
};

/// ECG code:
/// 0: N/A
/// 1: succeed measurement ECG
/// 45: wait ECG trigger (or not completed by unknown reason)
/// 41: failed by lead off
/// 42: failed by transmit overflow
/// 43: failed by low battery
/// 46: failed by manual stop
/// 47: failed by remote stop.
typedef NS_ENUM(NSInteger, ECGcode) {
    MicroLifeECGcodeNA = 0,
    MicroLifeECGcodeSucceedMeasurementECG = 1,
    MicroLifeECGcodeWaitECGTrigger = 45,
    MicroLifeECGcodeLeadOff = 41,
    MicroLifeECGcodeTransmitOverflow = 42,
    MicroLifeECGcodeLowBattery = 43,
    MicroLifeECGcodeManualStop = 46,
    MicroLifeECGcodeRemoteStop = 47
};

/// Specimen type:
/// 0: blood
/// 1: QC solution
typedef NS_ENUM(NSInteger, SpecimenType) {
    MicroLifeSpecimenTypeBlood = 0,
    MicroLifeSpecimenTypeQCSolution = 1
};

/// meal status:
/// 0: N/A
/// 1: Before Meal
/// 2: After Meal
typedef NS_ENUM(NSInteger, MealStatus) {
    MicroLifeMealStatusNA = 0,
    MicroLifeMealStatusBeforeMeal = 1,
    MicroLifeMealStatusAfterMeal = 2
};

/// MAP Type:
/// 0: Oscillometric MAP
/// 1: Formula MAP (MAP=1/3 SYS+ 2/3 DIA)
typedef NS_ENUM(NSInteger, MAPType) {
    MicroLifeMAPTypeOscillometric = 0,
    MicroLifeMAPTypeFormula = 1
};

NS_ASSUME_NONNULL_BEGIN

@interface MicroLifeDataModel : NSObject

@property (nonatomic, assign) DeviceType deviceType;

@property (nonatomic, assign) NSInteger CMD;

@property (nonatomic, assign) NSInteger indexYear;

@property (nonatomic, strong) NSString *dataValue;

@property (nonatomic, strong) NSNumber *success;

@property (nonatomic, strong) NSNumber *null;

@property (nonatomic, strong) NSDate *measureDate;

@property (nonatomic, strong) NSString *UUID;

@property (nonatomic, strong) NSString *Mac;

@property (nonatomic, strong) NSNumber *protocolID;

#pragma mark -
+ (instancetype)stringWithValue:(id)value;

/// set dataValue
/// @param value value
- (void)data:(id)value;

#pragma mark - Conversion
/// hexadecimal to decimal
/// @param hexadecimal hexadecimal
+ (NSNumber *)hexadecimal2Decimal:(NSString *)hexadecimal;

/// decimal to binary
/// @param decimall decimall
/// @param length length
+ (NSString *)decimall2Binary:(NSInteger)decimall length:(NSInteger)length;

/// binary to decimal
/// @param binary binary
+ (NSNumber *)binary2Decimall:(NSString *)binary;

/// hex to text
/// @param hexadecimal hexadecimal
/// @param encoding encoding
+ (NSString *)hexadecimal2String:(NSString *)hexadecimal cStringUsingEncoding:(NSStringEncoding)encoding ;

/// text to hexadecimal
/// @param valueStr valueStr
/// @param encoding encoding
+ (NSString *)string2Hexadecimal:(NSString *)valueStr cStringUsingEncoding:(NSStringEncoding)encoding;

/// byte inversion
/// @param high High
/// @param low low
+ (NSInteger)mixHigh:(NSInteger)high Low:(NSInteger)low;

/// byte inversion
/// @param value value
+ (NSString *)inversionValue:(NSInteger)value;

/// parse Date ,[Note] When there is no value, output the current time
/// @param dateString dateString
+ (NSDate *)parseDate:(NSString *)dateString;

/// parse MacAddress style ex. 00:11:22:33:44:55
/// @param mac mac
+ (NSString *)parseMacAddress:(NSData *)mac;

/// check Mac
/// 0: Mac None
/// 1: Mac format is correct
/// 2: Mac format error
/// @param mac mac
+ (NSInteger)checkMac:(NSData *)mac;

#pragma mark - Parsing tool
/// cut
/// @param offset offset
- (NSString *)cut:(NSInteger)offset;

/// parse Hexadecimal2Decimal
/// @param offset offset
- (NSNumber *)parseDecimal:(NSInteger)offset;

/// parse Binary2Decimal
/// @param offset offset
- (NSNumber *)parseDecimalby2:(NSInteger)offset;

/// parse Hexadecimal2Binary
/// [Note] conversion value to use >> parseDecimalby2:
/// @param offset offset
/// @param length length
- (NSString *)parseBinary:(NSInteger)offset length:(NSInteger)length;

/// parse Hexadecimal2String
/// @param offset offset
- (NSString *)parseString:(NSInteger)offset;

/// parse String2Hexadecimal
/// @param offset offset
- (NSString *)parseHexadecimal:(NSInteger)offset;

/// parse Year
/// @param offset offset
- (NSNumber *)parseYear:(NSInteger)offset;

#pragma mark - Command analysis of each device (rewrite by device)
/// <#Description#>
/// @param deviceType <#deviceType description#>
/// @param CMD <#CMD description#>
/// @param value <#value description#>
/// @param UUID <#UUID description#>
/// @param mac <#mac description#>
/// @param protocolID <#protocolID description#>
- (void)analysis:(DeviceType)deviceType CMD:(NSInteger)CMD Data:(id)value UUID:(NSString *)UUID Mac:(NSData *)mac ProtocolID:(NSNumber *)protocolID;

/// <#Description#>
/// @param replyACK <#replyACK description#>
/// @param replyNullACK <#replyNullACK description#>
- (void)replyACK:(BOOL)replyACK NullACK:(BOOL)replyNullACK;

/// <#Description#>
/// @param indexYear <#yearIndex description#>
- (void)indexYear:(NSUInteger)indexYear;

#pragma mark - Dictionary object conversion
/// Dictionary to model.
/// @param value Dictionary (NSDictionary) or json format character.
+ (instancetype)parseObjectWithKeyValues:(id)value;
+ (NSArray* _Nonnull)parseObjectArrayWithKeyValuesArray:(NSArray* const _Nonnull)array;

/// to Dictionary.
- (NSMutableDictionary *)parseDictionary;

//Get the type of a model object in an array
- (NSDictionary *)gainClassTypes;
@end

NS_ASSUME_NONNULL_END
